@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,21 +14,20 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 void loadMusic(char *);
-void freeAllMusic(void);
 void freeMusic(void);
-void playMusic(void);
-void playBossMusic(void);
+void playMapMusic(void);
+void playBossMusic(char *);
+void playDefaultBossMusic(void);
 void stopMusic(void);
-void pauseMusic(void);
-void loadBossMusic(char *);
+void pauseMusic(int);
 void fadeOutMusic(int);
 void fadeInMusic(int);
-void setMusicVolume(void);
 void resumeMusic(void);
 void fadeBossMusic(void);
 void loadGameOverMusic(void);
-void freeBossMusic(void);
+void playLoadedMusic(void);
+void updateMusicVolume(void);
