@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,34 +14,33 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 #include "../headers.h"
 
-#include "../graphics/animation.h"
-#include "../system/properties.h"
-#include "../entity.h"
-#include "../system/random.h"
 #include "../audio/audio.h"
-#include "../graphics/graphics.h"
-#include "../custom_actions.h"
-#include "../map.h"
-#include "../game.h"
 #include "../audio/music.h"
 #include "../collisions.h"
-#include "../item/key_items.h"
-#include "../world/target.h"
-#include "../graphics/decoration.h"
-#include "../player.h"
+#include "../credits.h"
+#include "../custom_actions.h"
+#include "../entity.h"
+#include "../game.h"
 #include "../geometry.h"
+#include "../graphics/animation.h"
+#include "../graphics/decoration.h"
 #include "../hud.h"
+#include "../item/key_items.h"
+#include "../map.h"
+#include "../player.h"
 #include "../projectile.h"
+#include "../system/error.h"
+#include "../system/properties.h"
+#include "../system/random.h"
+#include "../world/target.h"
 
 extern Entity *self, player;
 
-static int bodyParts = 10;
-
 static void bodyWait(void);
 static void initialise(void);
 static void headWait(void);
@@ -64,10 +63,11 @@ static void takeDamage(Entity *, int);
 static void shotAttackInit(void);
 static void shotAttackWindUp(void);
 static void shotAttack(void);
+static void specialShotMove(void);
 static void specialShotWait(void);
-static void specialShotBlock(void);
+static void specialShotBlock(Entity *);
 static void specialShotTouch(Entity *);
-static void biteReactToBlock(void);
+static void biteReactToBlock(Entity *);
 static void crushAttackInit(void);
 static void crushAttackMoveToPosition(void);
 static void crushAttack(void);
@@ -76,6 +76,9 @@ static void addSmokeAlongBody(void);
 static void stunned(void);
 static void crushAttackHit(Entity *);
 static void bodyTakeDamage(Entity *, int);
+static void starWait(void);
+static void creditsMove(void);
+static void creditsMoveOffScreen(void);
 
 Entity *addSnakeBoss(int x, int y, char *name)
 {
@@ -85,9 +88,7 @@ Entity *addSnakeBoss(int x, int y, char *name)
 
 	if (head == NULL)
 	{
-		printf("No free slots to add a Snake Boss\n");
-
-		exit(1);
+		showErrorAndExit("No free slots to add the Snake Boss");
 	}
 
 	loadProperties(name, head);
@@ -102,28 +103,41 @@ Entity *addSnakeBoss(int x, int y, char *name)
 	head->die = &die;
 	head->takeDamage = &takeDamage;
 
+	head->creditsAction = &creditsMove;
+
 	head->type = ENEMY;
 
 	head->active = FALSE;
 
-	setEntityAnimation(head, STAND);
+	setEntityAnimation(head, "STAND");
 
 	return head;
 }
 
 static void bodyWait()
 {
+	if (self->head->flags & FLASH)
+	{
+		self->flags |= FLASH;
+	}
+
+	else
+	{
+		self->flags &= ~FLASH;
+	}
+
 	checkToMap(self);
+
+	if (self->head->inUse == FALSE)
+	{
+		self->inUse = FALSE;
+	}
 }
 
 static void initialise()
 {
-	int minX, minY;
 	Target *t;
 
-	minX = getMapStartX();
-	minY = getMapStartY();
-
 	self->flags |= NO_DRAW;
 
 	if (self->active == TRUE)
@@ -138,9 +152,7 @@ static void initialise()
 
 			if (t == NULL)
 			{
-				printf("Snake boss is missing target\n");
-
-				exit(1);
+				showErrorAndExit("Snake boss cannot find target");
 			}
 
 			setSnakePosition(t->x, t->y);
@@ -149,11 +161,17 @@ static void initialise()
 
 			self->thinkTime = 60;
 
+			self->face = LEFT;
+
+			self->flags |= LIMIT_TO_SCREEN;
+
 			self->action = &riseUp;
 
-			playBossMusic();
+			playDefaultBossMusic();
 
 			initBossHealthBar();
+
+			setContinuePoint(FALSE, self->name, NULL);
 		}
 	}
 }
@@ -204,13 +222,18 @@ static void headWait()
 			break;
 		}
 	}
+
+	if (prand() % 180 == 0)
+	{
+		playSoundToMap("sound/boss/snake_boss/hiss", BOSS_CHANNEL, self->x, self->y, 0);
+	}
 }
 
 static void riseUp()
 {
 	Entity *e, *smoke;
 
-	facePlayer();
+	/*facePlayer();*/
 
 	self->thinkTime--;
 
@@ -266,28 +289,22 @@ static void createBody()
 	int i;
 	Entity **body, *head;
 
-	body = (Entity **)malloc(bodyParts * sizeof(Entity *));
+	body = malloc(self->mental * sizeof(Entity *));
 
 	if (body == NULL)
 	{
-		printf("Failed to allocate a whole %d bytes for Snake Boss body...\n", bodyParts * (int)sizeof(Entity *));
-
-		exit(1);
+		showErrorAndExit("Failed to allocate a whole %d bytes for Snake Boss body...", self->mental * (int)sizeof(Entity *));
 	}
 
 	snprintf(bodyName, sizeof(bodyName), "%s_body", self->name);
 
-	/* Create in reverse order so that it is draw correctly */
-
-	for (i=bodyParts-1;i>=0;i--)
+	for (i=self->mental-1;i>=0;i--)
 	{
 		body[i] = getFreeEntity();
 
 		if (body[i] == NULL)
 		{
-			printf("No free slots to add a Snake Boss body part\n");
-
-			exit(1);
+			showErrorAndExit("No free slots to add a Snake Boss body part");
 		}
 
 		loadProperties(bodyName, body[i]);
@@ -302,13 +319,15 @@ static void createBody()
 		body[i]->die = &entityDieNoDrop;
 		body[i]->takeDamage = &bodyTakeDamage;
 
+		body[i]->creditsAction = &bodyWait;
+
 		body[i]->type = ENEMY;
 
 		body[i]->active = FALSE;
 
 		body[i]->flags |= NO_DRAW;
 
-		setEntityAnimation(body[i], STAND);
+		setEntityAnimation(body[i], "STAND");
 	}
 
 	/* Recreate the head so that it's on top */
@@ -317,7 +336,7 @@ static void createBody()
 
 	if (head == NULL)
 	{
-		printf("No free slots to add a Snake Boss head\n");
+		showErrorAndExit("No free slots to add a Snake Boss head");
 	}
 
 	*head = *self;
@@ -328,7 +347,7 @@ static void createBody()
 
 	/* Link the sections */
 
-	for (i=bodyParts-1;i>=0;i--)
+	for (i=self->mental-1;i>=0;i--)
 	{
 		if (i == 0)
 		{
@@ -348,9 +367,9 @@ static void createBody()
 
 static void biteAttackInit()
 {
-	facePlayer();
+	/*facePlayer();*/
 
-	setEntityAnimation(self, ATTACK_1);
+	setEntityAnimation(self, "ATTACK_1");
 
 	self->targetX = self->endX + (self->face == LEFT ? -32 : 32);
 	self->targetY = self->endY - 32;
@@ -369,18 +388,15 @@ static void biteAttackWindUp()
 {
 	checkToMap(self);
 
-	if (fabs(self->targetX - self->x) <= fabs(self->dirX) && fabs(self->targetY - self->y) <= fabs(self->dirY))
+	if (atTarget())
 	{
-		self->targetX = player.x;
-
-		if (abs(self->x - self->targetX) > 320)
-		{
-			self->targetX = self->face == RIGHT ? self->x + 320 - self->w - 1 : self->x - 320;
-		}
+		self->targetX = self->face == RIGHT ? self->x + 320 - self->w - 1 : self->x - 320;
 
 		self->action = &biteAttack;
 
 		self->reactToBlock = &biteReactToBlock;
+
+		self->dirX = (self->targetX < self->x ? -self->speed * 2 : self->speed * 2);
 	}
 
 	alignBodyToHead();
@@ -388,8 +404,6 @@ static void biteAttackWindUp()
 
 static void biteAttack()
 {
-	self->dirX = (self->targetX < self->x ? -self->speed * 2 : self->speed * 2);
-
 	checkToMap(self);
 
 	if (fabs(self->x - self->targetX) < self->speed)
@@ -422,9 +436,9 @@ static void biteAttack()
 
 static void shotAttackInit()
 {
-	facePlayer();
+	/*facePlayer();*/
 
-	setEntityAnimation(self, ATTACK_1);
+	setEntityAnimation(self, "ATTACK_1");
 
 	self->targetX = self->endX + (self->face == LEFT ? -50 : 50);
 	self->targetY = self->endY - 32;
@@ -437,13 +451,15 @@ static void shotAttackInit()
 	self->action = &shotAttackWindUp;
 
 	self->flags |= UNBLOCKABLE;
+
+	self->startX = 0;
 }
 
 static void shotAttackWindUp()
 {
 	checkToMap(self);
 
-	if (fabs(self->targetX - self->x) <= fabs(self->dirX) && fabs(self->targetY - self->y) <= fabs(self->dirY))
+	if (atTarget())
 	{
 		self->maxThinkTime = 5;
 
@@ -463,11 +479,17 @@ static void shotAttack()
 
 	if (self->thinkTime <= 0)
 	{
-		if (prand() % 10 == 0)
+		playSoundToMap("sound/boss/snake_boss/snake_boss_shot", BOSS_CHANNEL, self->x, self->y, 0);
+
+		if (prand() % 4 == 0 && self->startX == 0)
 		{
 			e = addProjectile("boss/snake_boss_special_shot", self, self->x + self->w / 2, self->y + self->h / 2, (self->face == RIGHT ? 7 : -7), 0);
 
+			e->action = &specialShotMove;
+
 			e->reactToBlock = &specialShotBlock;
+
+			self->startX = 1;
 		}
 
 		else
@@ -533,13 +555,14 @@ static void changeSidesInit()
 
 static void changeSides()
 {
+	int side;
 	Entity *e, *smoke;
 	Target *t;
 
 	self->x += self->dirX;
 	self->y += self->dirY;
 
-	if (fabs(self->targetX - self->x) <= fabs(self->dirX) && fabs(self->targetY - self->y) <= fabs(self->dirY))
+	if (atTarget())
 	{
 		self->flags |= NO_DRAW;
 
@@ -551,13 +574,25 @@ static void changeSides()
 
 		self->thinkTime = 120;
 
-		t = getTargetByName(prand() % 2 == 0 ? "SNAKE_BOSS_TARGET_RIGHT" : "SNAKE_BOSS_TARGET_LEFT");
+		side = prand() % 2;
+
+		if (side == 0)
+		{
+			t = getTargetByName("SNAKE_BOSS_TARGET_LEFT");
+
+			self->face = RIGHT;
+		}
+
+		else
+		{
+			t = getTargetByName("SNAKE_BOSS_TARGET_RIGHT");
+
+			self->face = LEFT;
+		}
 
 		if (t == NULL)
 		{
-			printf("Snake boss is missing target\n");
-
-			exit(1);
+			showErrorAndExit("Snake boss cannot find target");
 		}
 
 		self->damage = 0;
@@ -614,7 +649,7 @@ static void crushAttackMoveToPosition()
 
 	checkToMap(self);
 
-	if (fabs(self->targetX - self->x) <= fabs(self->dirX) && fabs(self->targetY - self->y) <= fabs(self->dirY))
+	if (atTarget())
 	{
 		self->y = self->targetY;
 
@@ -710,7 +745,7 @@ static void riseUpWait()
 
 static void attackFinished()
 {
-	setEntityAnimation(self, STAND);
+	setEntityAnimation(self, "STAND");
 
 	self->flags &= ~UNBLOCKABLE;
 
@@ -731,7 +766,7 @@ static void returnToStart()
 {
 	checkToMap(self);
 
-	if (fabs(self->targetX - self->x) <= fabs(self->dirX) && fabs(self->targetY - self->y) <= fabs(self->dirY))
+	if (atTarget())
 	{
 		self->y = self->targetY;
 
@@ -758,8 +793,8 @@ static void alignBodyToHead()
 	partDistanceX = self->endX - self->x;
 	partDistanceY = fabs(self->endY - self->y);
 
-	partDistanceX /= bodyParts;
-	partDistanceY /= bodyParts;
+	partDistanceX /= self->mental;
+	partDistanceY /= self->mental;
 
 	e = self->target;
 
@@ -785,23 +820,13 @@ static void alignBodyToHead()
 			e->flags &= ~NO_DRAW;
 		}
 
-		if (self->flags & FLASH)
-		{
-			e->flags |= FLASH;
-		}
-
-		else
-		{
-			e->flags &= ~FLASH;
-		}
-
 		e = e->target;
 	}
 }
 
 static void takeDamage(Entity *other, int damage)
 {
-	int i;
+	Entity *temp;
 
 	if (!(self->flags & INVULNERABLE))
 	{
@@ -822,35 +847,22 @@ static void takeDamage(Entity *other, int damage)
 
 		else
 		{
-			self->health -= (self->flags & HELPLESS ? damage * 3 : damage);
+			damage = (self->flags & HELPLESS ? damage * 3 : damage);
+
+			self->health -= damage;
 		}
 
 		if (self->health > 0)
 		{
-			setCustomAction(self, &flashWhite, 6, 0);
-			setCustomAction(self, &invulnerableNoFlash, 20, 0);
+			setCustomAction(self, &flashWhite, 6, 0, 0);
+			setCustomAction(self, &invulnerableNoFlash, HIT_INVULNERABLE_TIME, 0, 0);
 
-			i = prand() % 3;
-
-			switch (i)
-			{
-				case 0:
-					playSoundToMap("sound/common/splat1.ogg", -1, self->x, self->y, 0);
-				break;
-
-				case 1:
-					playSoundToMap("sound/common/splat2.ogg", -1, self->x, self->y, 0);
-				break;
-
-				default:
-					playSoundToMap("sound/common/splat3.ogg", -1, self->x, self->y, 0);
-				break;
-			}
+			enemyPain();
 		}
 
 		else
 		{
-			setEntityAnimation(self, PAIN);
+			setEntityAnimation(self, "PAIN");
 
 			self->health = 0;
 
@@ -872,6 +884,17 @@ static void takeDamage(Entity *other, int damage)
 
 			self->flags &= ~(HELPLESS|FLY);
 		}
+
+		if (other->type == PROJECTILE)
+		{
+			temp = self;
+
+			self = other;
+
+			self->die();
+
+			self = temp;
+		}
 	}
 }
 
@@ -885,7 +908,7 @@ static void die()
 
 	if (self->flags & FLY)
 	{
-		if (fabs(self->targetX - self->x) <= fabs(self->dirX) && fabs(self->targetY - self->y) <= fabs(self->dirY))
+		if (atTarget())
 		{
 			self->dirX = self->dirY = 0;
 
@@ -929,20 +952,24 @@ static void fallToGround()
 
 	self->thinkTime--;
 
-	if (self->thinkTime <= 0)
+	if (self->thinkTime == 0)
 	{
+		playSoundToMap("sound/boss/snake_boss/snake_boss_die", BOSS_CHANNEL, self->x, self->y, 0);
+
 		self->flags &= ~FLY;
 
 		self->dirX = self->face == LEFT ? -8 : 8;
 
-		setEntityAnimation(self, DIE);
+		setEntityAnimation(self, "DIE");
 	}
 
 	checkToMap(self);
 
 	if (self->flags & ON_GROUND)
 	{
-		setEntityAnimation(self, PAIN);
+		playSoundToMap("sound/common/crash", BOSS_CHANNEL, self->x, self->y, 0);
+
+		setEntityAnimation(self, "PAIN");
 
 		self->thinkTime = 180;
 
@@ -982,6 +1009,10 @@ static void dieWait()
 
 	if (self->thinkTime <= 0)
 	{
+		clearContinuePoint();
+
+		increaseKillCount();
+
 		freeBossHealthBar();
 
 		e = addKeyItem("item/heart_container", self->x + self->w / 2, self->y);
@@ -1005,25 +1036,37 @@ static void dieWait()
 	}
 }
 
-static void specialShotBlock()
+static void specialShotMove()
 {
-	self->dirX *= -1;
+	if (!(self->flags & FLY))
+	{
+		self->dirX = (self->face == LEFT ? 5 : -5);
 
-	self->type = ENEMY;
+		self->dirY = -5;
 
-	self->target = self->parent;
+		self->type = ENEMY;
 
-	self->parent = NULL;
+		self->target = self->parent;
+
+		self->parent = NULL;
+
+		self->action = &specialShotWait;
+
+		self->touch = NULL;
+
+		self->thinkTime = 300;
+
+		self->die = &entityDieNoDrop;
+	}
+
+	checkToMap(self);
+}
+
+static void specialShotBlock(Entity *other)
+{
+	self->dirX = 0;
 
 	self->flags &= ~FLY;
-
-	self->action = &specialShotWait;
-
-	self->touch = NULL;
-
-	self->thinkTime = 300;
-
-	self->die = &entityDieNoDrop;
 }
 
 static void specialShotWait()
@@ -1069,34 +1112,22 @@ static void specialShotTouch(Entity *other)
 	}
 }
 
-static void biteReactToBlock()
+static void biteReactToBlock(Entity *other)
 {
-	self->maxThinkTime--;
+	self->dirX = 0;
 
-	if (self->maxThinkTime <= 0)
-	{
-		self->action = &attackFinished;
-	}
+	self->targetX = self->x;
 
-	else
-	{
-		self->targetX = self->endX + (self->face == LEFT ? -32 : 32);
-		self->targetY = self->endY - 32;
-
-		calculatePath(self->x, self->y, self->targetX, self->targetY, &self->dirX, &self->dirY);
-
-		self->dirX *= 4;
-		self->dirY *= 4;
-
-		self->action = &biteAttackWindUp;
-	}
+	self->x = (int)self->x;
 }
 
 static void stunned()
 {
+	int i;
 	long onGround = self->flags & ON_GROUND;
+	Entity *e;
 
-	setEntityAnimation(self, PAIN);
+	setEntityAnimation(self, "PAIN");
 
 	checkToMap(self);
 
@@ -1104,7 +1135,7 @@ static void stunned()
 	{
 		self->dirX = 0;
 
-		setCustomAction(self, &helpless, 300, 0);
+		setCustomAction(self, &helpless, 300, 0, 0);
 
 		if (onGround == 0)
 		{
@@ -1112,6 +1143,37 @@ static void stunned()
 		}
 
 		self->action = &attackFinished;
+
+		for (i=0;i<2;i++)
+		{
+			e = getFreeEntity();
+
+			if (e == NULL)
+			{
+				showErrorAndExit("No free slots to add the Armour Boss's Star");
+			}
+
+			loadProperties("boss/armour_boss_star", e);
+
+			e->x = self->x;
+			e->y = self->y;
+
+			e->action = &starWait;
+
+			e->draw = &drawLoopingAnimationToMap;
+
+			e->thinkTime = 300;
+
+			e->head = self;
+
+			setEntityAnimation(e, "STAND");
+
+			e->currentFrame = (i == 0 ? 0 : 6);
+
+			e->x = self->x + self->w / 2 - e->w / 2;
+
+			e->y = self->y - e->h - 8;
+		}
 	}
 
 	alignBodyToHead();
@@ -1130,7 +1192,7 @@ static void addSmokeAlongBody()
 		addSmoke((self->face == LEFT ? self->x : self->endX) + (prand() % bodyLength), self->y + prand() % self->h, "decoration/dust");
 	}
 
-	playSoundToMap("sound/common/crash.ogg", BOSS_CHANNEL, self->x, self->y, 0);
+	playSoundToMap("sound/common/crash", BOSS_CHANNEL, self->x, self->y, 0);
 }
 
 static void bodyTakeDamage(Entity *other, int damage)
@@ -1152,3 +1214,52 @@ static void bodyTakeDamage(Entity *other, int damage)
 		self = temp;
 	}
 }
+
+static void starWait()
+{
+	self->thinkTime--;
+
+	if (self->thinkTime <= 0 || self->head->health <= 0)
+	{
+		self->inUse = FALSE;
+	}
+}
+
+static void creditsMove()
+{
+	if (self->health != -1)
+	{
+		createBody();
+
+		self->endY = self->y + 32;
+
+		self->y -= 32;
+
+		self->health = -1;
+	}
+
+	bossMoveToMiddle();
+
+	self->endX = self->x;
+
+	alignBodyToHead();
+
+	if (self->thinkTime <= 0)
+	{
+		self->creditsAction = &creditsMoveOffScreen;
+	}
+}
+
+static void creditsMoveOffScreen()
+{
+	self->x -= 20;
+
+	if (self->x <= -self->w)
+	{
+		self->inUse = FALSE;
+	}
+
+	self->endX = self->x;
+
+	alignBodyToHead();
+}
