@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,19 +14,20 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 #include "headers.h"
 
-#include "collisions.h"
-#include "map.h"
 #include "audio/audio.h"
+#include "collisions.h"
+#include "entity.h"
+#include "map.h"
 
-extern Entity entity[MAX_ENTITIES], *self;
+extern Entity *self;
 extern Entity player, playerShield, playerWeapon;
 
-static Grid grid[GRID_HEIGHT][GRID_WIDTH];
+static Grid grid[GRID_MAX_Y][GRID_MAX_X];
 
 static void addToList(int, int, Entity *);
 
@@ -34,9 +35,9 @@ void initCollisionGrid()
 {
 	int x, y;
 
-	for (y=0;y<GRID_HEIGHT;y++)
+	for (y=0;y<GRID_MAX_Y;y++)
 	{
-		for (x=0;x<GRID_WIDTH;x++)
+		for (x=0;x<GRID_MAX_X;x++)
 		{
 			grid[y][x].count = 0;
 			grid[y][x].listHead.next = NULL;
@@ -46,50 +47,41 @@ void initCollisionGrid()
 
 void addToGrid(Entity *e)
 {
-	int left, right, top, bottom;
+	int left, right, top, bottom, x, y;
 
 	if (e == &playerWeapon)
 	{
 		if (e->face == LEFT)
 		{
-			left = (e->x + e->parent->w - e->offsetX) / TILE_SIZE / GRID_COUNT;
-			right = (e->x + e->parent->w - e->w - e->offsetX) / TILE_SIZE / GRID_COUNT;
+			left = (e->x - e->w + e->offsetX - 1) / TILE_SIZE / GRID_SIZE;
+			right = (e->x + e->offsetX - 1) / TILE_SIZE / GRID_SIZE;
 		}
 
 		else
 		{
-			left = (e->x + e->offsetX) / TILE_SIZE / GRID_COUNT;
-			right = (e->x + e->offsetX + e->w - 1) / TILE_SIZE / GRID_COUNT;
+			left = (e->x + e->offsetX) / TILE_SIZE / GRID_SIZE;
+			right = (e->x + e->offsetX + e->w) / TILE_SIZE / GRID_SIZE;
 		}
 
-		top = e->y / TILE_SIZE / GRID_COUNT;
-		bottom = (e->y + e->h - 1) / TILE_SIZE / GRID_COUNT;
+		top = e->y / TILE_SIZE / GRID_SIZE;
+		bottom = (e->y + e->h) / TILE_SIZE / GRID_SIZE;
 	}
 
 	else
 	{
-		left = (e->x + e->box.x) / TILE_SIZE / GRID_COUNT;
-		right = (e->x + e->box.x + e->box.w - 1) / TILE_SIZE / GRID_COUNT;
+		left = getLeftEdge(e) / TILE_SIZE / GRID_SIZE;
+		right = getRightEdge(e) / TILE_SIZE / GRID_SIZE;
 
-		top = (e->y + e->box.y) / TILE_SIZE / GRID_COUNT;
-		bottom = (e->y + e->box.y + e->box.h - 1) / TILE_SIZE / GRID_COUNT;
+		top = (e->y + e->box.y) / TILE_SIZE / GRID_SIZE;
+		bottom = (e->y + e->box.y + e->box.h) / TILE_SIZE / GRID_SIZE;
 	}
 
-	addToList(top, left, e);
-
-	if (left != right)
+	for (x=left;x<=right;x++)
 	{
-		addToList(top, right, e);
-	}
-
-	if (top != bottom)
-	{
-		addToList(bottom, left, e);
-	}
-
-	if (left != right && top != bottom)
-	{
-		addToList(bottom, right, e);
+		for (y=top;y<=bottom;y++)
+		{
+			addToList(y, x, e);
+		}
 	}
 }
 
@@ -97,7 +89,7 @@ static void addToList(int y, int x, Entity *e)
 {
 	EntityList *listHead, *list;
 
-	if (x < 0 || x >= GRID_WIDTH || y < 0 || y >= GRID_HEIGHT)
+	if (x < 0 || x >= GRID_MAX_X || y < 0 || y >= GRID_MAX_Y)
 	{
 		return;
 	}
@@ -109,7 +101,7 @@ static void addToList(int y, int x, Entity *e)
 		listHead = listHead->next;
 	}
 
-	list = (EntityList *)malloc(sizeof(EntityList));
+	list = malloc(sizeof(EntityList));
 
 	list->entity = e;
 	list->next = NULL;
@@ -124,9 +116,9 @@ void freeCollisionGrid()
 	int x, y;
 	EntityList *p, *q;
 
-	for (y=0;y<GRID_HEIGHT;y++)
+	for (y=0;y<GRID_MAX_Y;y++)
 	{
-		for (x=0;x<GRID_WIDTH;x++)
+		for (x=0;x<GRID_MAX_X;x++)
 		{
 			for (p=grid[y][x].listHead.next;p!=NULL;p=q)
 			{
@@ -147,14 +139,19 @@ void doCollisions()
 	Entity *e1, *e2, *temp;
 	EntityList *list1, *list2;
 
-	for (i=0;i<GRID_HEIGHT;i++)
+	for (i=0;i<GRID_MAX_Y;i++)
 	{
-		for (j=0;j<GRID_WIDTH;j++)
+		for (j=0;j<GRID_MAX_X;j++)
 		{
 			for (list1=grid[i][j].listHead.next;list1!=NULL;list1=list1->next)
 			{
 				e1 = list1->entity;
 
+				if (e1->flags & TELEPORTING)
+				{
+					continue;
+				}
+
 				if (e1->inUse == TRUE && e1->touch != NULL)
 				{
 					for (list2=grid[i][j].listHead.next;list2!=NULL;list2=list2->next)
@@ -167,11 +164,23 @@ void doCollisions()
 							continue;
 						}
 
+						if ((e1->type == CONVEYOR_BELT && e2->type == CONVEYOR_BELT) ||
+							(e1->type == WEAK_WALL && e2->type == WEAK_WALL) ||
+							(e1->type == ANTI_GRAVITY && e2->type == ANTI_GRAVITY))
+						{
+							continue;
+						}
+
+						if (e2->flags & TELEPORTING)
+						{
+							continue;
+						}
+
 						if (e1 != e2 && e2->inUse == TRUE && e2->touch != NULL)
 						{
 							if (e1->type == PROJECTILE)
 							{
-								if (e2->type == PROJECTILE || (e1->parent != NULL && e1->parent->type == ENEMY && e2->type == ENEMY))
+								if (e2->type == PROJECTILE || (e1->parent != NULL && (e1->parent->type == ENEMY || e1->parent->type == SPAWNER) && e2->type == ENEMY))
 								{
 									continue;
 								}
@@ -182,30 +191,14 @@ void doCollisions()
 								continue;
 							}
 
-							if (e1->face == RIGHT)
-							{
-								x1 = e1->x + e1->box.x;
-							}
-
-							else
-							{
-								x1 = e1->x + e1->w - e1->box.w - e1->box.x;
-							}
+							x1 = getLeftEdge(e1);
 
 							y1 = e1->y + e1->box.y;
 
 							w1 = e1->box.w;
 							h1 = e1->box.h;
 
-							if (e2->face == RIGHT)
-							{
-								x2 = e2->x + e2->box.x;
-							}
-
-							else
-							{
-								x2 = e2->x + e2->w - e2->box.w - e2->box.x;
-							}
+							x2 = getLeftEdge(e2);
 
 							y2 = e2->y + e2->box.y;
 
@@ -214,21 +207,34 @@ void doCollisions()
 
 							if (e1 == &playerWeapon)
 							{
-								x1 = e1->x + e1->box.x;
-								y1 = e1->y + e1->box.y;
-
 								if (e1->face == LEFT)
 								{
-									x1 += e1->parent->w - e1->w - e1->offsetX;
+									x1 = e1->parent->x - e1->w + e1->offsetX - 1;
 								}
 
 								else
 								{
-									x1 += e1->offsetX;
+									x1 = e1->x + e1->offsetX;
 								}
 
 
-								y1 += e1->offsetY;
+								y1 = e1->parent->y + e1->offsetY;
+							}
+
+							else if (e2 == &playerWeapon)
+							{
+								if (e2->face == LEFT)
+								{
+									x2 = e2->parent->x - e2->w + e2->offsetX - 1;
+								}
+
+								else
+								{
+									x2 = e2->x + e2->offsetX;
+								}
+
+
+								y2 = e2->parent->y + e2->offsetY;
 							}
 
 							if (collision(x1, y1, w1, h1, x2, y2, w2, h2) == TRUE)
@@ -242,6 +248,11 @@ void doCollisions()
 								self = temp;
 							}
 						}
+
+						if (e1->inUse == FALSE)
+						{
+							break;
+						}
 					}
 				}
 			}
@@ -249,15 +260,15 @@ void doCollisions()
 	}
 }
 
-void checkEntityToEntity(Entity *e)
+Entity *checkEntityToEntity(Entity *e)
 {
 	int i, j, x1, y1, x2, y2, w1, h1, w2, h2;
-	Entity *e1, *e2, *temp;
+	Entity *e1, *e2;
 	EntityList *list1, *list2;
 
-	for (i=0;i<GRID_HEIGHT;i++)
+	for (i=0;i<GRID_MAX_Y;i++)
 	{
-		for (j=0;j<GRID_WIDTH;j++)
+		for (j=0;j<GRID_MAX_X;j++)
 		{
 			for (list1=grid[i][j].listHead.next;list1!=NULL;list1=list1->next)
 			{
@@ -274,13 +285,18 @@ void checkEntityToEntity(Entity *e)
 					{
 						e2 = list2->entity;
 
-						if (e1 != e2 && e2->inUse == TRUE && e2->touch != NULL)
+						if (e1 != e2 && e2->inUse == TRUE && e2->touch != NULL && e2->weight > 0)
 						{
 							if (e1->type == ENEMY && e2->type == ENEMY)
 							{
 								continue;
 							}
 
+							if (e1->type == ANTI_GRAVITY || e2->type == ANTI_GRAVITY)
+							{
+								continue;
+							}
+
 							if ((e1->type != PLAYER && (e2->flags & PLAYER_TOUCH_ONLY)) ||
 								((e1->flags & PLAYER_TOUCH_ONLY) && e2->type != PLAYER))
 							{
@@ -300,30 +316,14 @@ void checkEntityToEntity(Entity *e)
 								continue;
 							}
 
-							if (e1->face == RIGHT)
-							{
-								x1 = e1->x + e1->box.x;
-							}
-
-							else
-							{
-								x1 = e1->x + e1->w - e1->box.w - e1->box.x;
-							}
+							x1 = getLeftEdge(e1);
 
 							y1 = e1->y + e1->box.y;
 
 							w1 = e1->box.w;
 							h1 = e1->box.h;
 
-							if (e2->face == RIGHT)
-							{
-								x2 = e2->x + e2->box.x;
-							}
-
-							else
-							{
-								x2 = e2->x + e2->w - e2->box.w - e2->box.x;
-							}
+							x2 = getLeftEdge(e2);
 
 							y2 = e2->y + e2->box.y;
 
@@ -351,13 +351,7 @@ void checkEntityToEntity(Entity *e)
 
 							if (collision(x1, y1, w1, h1, x2, y2, w2, h2) == TRUE)
 							{
-								temp = self;
-
-								self = e1;
-
-								self->touch(e2);
-
-								self = temp;
+								return e2;
 							}
 						}
 					}
@@ -365,29 +359,51 @@ void checkEntityToEntity(Entity *e)
 			}
 		}
 	}
+
+	return NULL;
 }
 
 Entity *isSpaceEmpty(Entity *e)
 {
-	int i;
+	EntityList *el, *entities;
+
+	entities = getEntities();
 
 	if (player.inUse == TRUE && collision(e->x, e->y, e->w, e->h, player.x, player.y, player.w, player.h) == 1)
 	{
 		return &player;
 	}
 
-	for (i=0;i<MAX_ENTITIES;i++)
+	for (el=entities->next;el!=NULL;el=el->next)
 	{
-		if (entity[i].inUse == TRUE && e != &entity[i]
-			&& collision(e->x, e->y, e->w, e->h, entity[i].x, entity[i].y, entity[i].w, entity[i].h) == 1)
+		if (el->entity->inUse == TRUE && e != el->entity
+			&& collision(e->x, e->y, e->w, e->h, el->entity->x, el->entity->y, el->entity->w, el->entity->h) == 1)
 		{
-			return &entity[i];
+			return el->entity;
 		}
 	}
 
 	return NULL;
 }
 
+int isNearObstacle(Entity *e)
+{
+	EntityList *el, *entities;
+
+	entities = getEntities();
+
+	for (el=entities->next;el!=NULL;el=el->next)
+	{
+		if (el->entity->inUse == TRUE && (el->entity->flags & (OBSTACLE|PUSHABLE))
+			&& collision(e->x, e->y, e->w, e->h, el->entity->x, el->entity->y, el->entity->w, el->entity->h) == 1)
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 void checkToMap(Entity *e)
 {
 	int i, x1, x2, y1, y2, previousEnvironment;
@@ -452,7 +468,7 @@ void checkToMap(Entity *e)
 
 							e->y *= TILE_SIZE;
 
-							e->y -= e->h;
+							e->y -= e->h + 0.5;
 
 							e->dirY = 0;
 
@@ -576,7 +592,7 @@ void checkToMap(Entity *e)
 
 							e->y *= TILE_SIZE;
 
-							e->y -= e->h;
+							e->y -= e->h + 0.5;
 
 							e->dirY = 0;
 
@@ -817,6 +833,13 @@ void checkToMap(Entity *e)
 							e->flags |= ON_GROUND;
 						}
 					}
+
+					if (e->type == PROJECTILE)
+					{
+						e->die();
+
+						return;
+					}
 				}
 
 				else if ((bottomLeft != BLANK_TILE && bottomLeft < BACKGROUND_TILE_START) || (bottomRight != BLANK_TILE && bottomRight < BACKGROUND_TILE_START))
@@ -893,8 +916,10 @@ void checkToMap(Entity *e)
 	e->x += e->dirX;
 	e->y += e->dirY;
 
-	x1 = e->type == PLAYER ? getMinMapX() : 0;
-	y1 = e->type == PLAYER ? getMinMapY() : 0;
+	x1 = (e->type == PLAYER || (e->flags & LIMIT_TO_SCREEN)) ? getPlayerMinX() : getMapMinX();
+	x2 = (e->type == PLAYER || (e->flags & LIMIT_TO_SCREEN)) ? getPlayerMaxX() : getMapMaxX();
+
+	y1 = (e->flags & LIMIT_TO_SCREEN) ? getMapStartY() : getMapMinY() - 300;
 
 	if (e->x < x1)
 	{
@@ -914,9 +939,9 @@ void checkToMap(Entity *e)
 
 	}
 
-	else if (e->x + e->w >= maxMapX())
+	else if (e->x + e->w >= x2)
 	{
-		e->x = maxMapX() - e->w - 1;
+		e->x = x2 - e->w - 1;
 
 		e->dirX = 0;
 
@@ -931,17 +956,31 @@ void checkToMap(Entity *e)
 		}
 	}
 
-	if (e->y > maxMapY() && e->y - e->dirY <= maxMapY())
+	if (e->y > getMapMaxY() && e->y - e->dirY <= getMapMaxY())
 	{
-		e->flags &= ~HELPLESS|INVULNERABLE;
+		e->flags &= ~(HELPLESS|INVULNERABLE);
 
-		e->fallout();
+		if (e->fallout == NULL)
+		{
+			printf("%s has no fallout defined. Removing\n", e->name);
+
+			e->inUse = FALSE;
+
+			return;
+		}
+
+		else
+		{
+			e->fallout();
+		}
 	}
 
-	else if (e->y < -300)
+	else if (e->y < y1)
 	{
 		/* Way too high... */
 
+		e->y = y1;
+
 		e->dirY = 0;
 	}
 
@@ -969,7 +1008,10 @@ void checkToMap(Entity *e)
 
 			if (e->element != FIRE)
 			{
-				playSoundToMap("sound/common/lava.ogg", -1, self->x, self->y, 0);
+				if (e->type != TEMP_ITEM)
+				{
+					playSoundToMap("sound/common/lava", -1, e->x, e->y, 0);
+				}
 
 				e->flags &= ~(HELPLESS|INVULNERABLE);
 
@@ -978,10 +1020,10 @@ void checkToMap(Entity *e)
 		}
 	}
 
-	else if ((topLeft >= SLIME_TILE_START && topLeft <= SLIME_TILE_END) ||
-		(bottomLeft >= SLIME_TILE_START && bottomLeft <= SLIME_TILE_END) ||
-		(topRight >= SLIME_TILE_START && topRight <= SLIME_TILE_END) ||
-		(bottomRight >= SLIME_TILE_START && bottomRight <= SLIME_TILE_END))
+	else if ((topLeft >= SLIME_TILE_START && topLeft <= SLIME_TILE_BLEND) ||
+		(bottomLeft >= SLIME_TILE_START && bottomLeft <= SLIME_TILE_BLEND) ||
+		(topRight >= SLIME_TILE_START && topRight <= SLIME_TILE_BLEND) ||
+		(bottomRight >= SLIME_TILE_START && bottomRight <= SLIME_TILE_BLEND))
 	{
 		e->environment = SLIME;
 
@@ -991,7 +1033,10 @@ void checkToMap(Entity *e)
 
 			if (e->element != SLIME)
 			{
-				playSoundToMap("sound/common/slime.ogg", -1, self->x, self->y, 0);
+				if (e->type != TEMP_ITEM)
+				{
+					playSoundToMap("sound/common/slime", -1, e->x, e->y, 0);
+				}
 
 				e->flags &= ~(HELPLESS|INVULNERABLE);
 
@@ -1016,7 +1061,10 @@ void checkToMap(Entity *e)
 
 			if (previousEnvironment != WATER && e->fallout != NULL)
 			{
-				playSoundToMap("sound/common/splash.ogg", -1, self->x, self->y, 0);
+				if (e->type != TEMP_ITEM)
+				{
+					playSoundToMap("sound/common/splash", -1, e->x, e->y, 0);
+				}
 
 				if (!(e->flags & FLOATS))
 				{
@@ -1032,8 +1080,11 @@ void checkToMap(Entity *e)
 int isAtEdge(Entity *e)
 {
 	int i, tile;
-	int x = e->x + (e->dirX > 0 ? e->w : 0);
+	int x = e->face == LEFT ? floor(e->x) : ceil(e->x) + e->w;
 	int y = e->y + e->h - 1;
+	EntityList *el, *entities;
+
+	entities = getEntities();
 
 	x /= TILE_SIZE;
 	y /= TILE_SIZE;
@@ -1110,13 +1161,22 @@ int isAtEdge(Entity *e)
 		}
 	}
 
+	x = e->face == LEFT ? floor(e->x) : ceil(e->x);
+
+	if (e->face == RIGHT)
+	{
+		x += e->w;
+	}
+
 	/* There might still be Entities that can be walked on */
 
-	for (i=0;i<MAX_ENTITIES;i++)
+	for (el=entities->next;el!=NULL;el=el->next)
 	{
-		if (e != &entity[i] && entity[i].inUse == TRUE && ((entity[i].flags & PUSHABLE) || (entity[i].type == WEAK_WALL)))
+		if (e != el->entity && el->entity->inUse == TRUE && el->entity->touch != NULL
+			&& ((el->entity->flags & (PUSHABLE|OBSTACLE)) || (el->entity->type == WEAK_WALL)
+			|| (el->entity->type == PRESSURE_PLATE) || (el->entity->type == ANTI_GRAVITY)))
 		{
-			if (collision(e->x + (e->face == LEFT ? 0 : e->w), e->y + e->h, 1, 128, entity[i].x, entity[i].y, entity[i].w, entity[i].h) == TRUE)
+			if (collision(x, e->y, 1, e->h + 10, el->entity->x, el->entity->y, el->entity->w, el->entity->h) == TRUE)
 			{
 				return FALSE;
 			}
@@ -1126,10 +1186,141 @@ int isAtEdge(Entity *e)
 	return TRUE;
 }
 
+int isAtCeilingEdge(Entity *e)
+{
+	int i, tile;
+	int x = e->x + (e->dirX > 0 ? e->w : 0);
+	int y = e->y;
+
+	x /= TILE_SIZE;
+	y /= TILE_SIZE;
+
+	y--;
+
+	tile = mapTileAt(x, y);
+
+	/* Return immediately if the tile isn't blank */
+
+	if (tile != BLANK_TILE && tile < BACKGROUND_TILE_START)
+	{
+		return FALSE;
+	}
+
+	if (e->w > TILE_SIZE)
+	{
+		if (e->dirX > 0)
+		{
+			for (i=0;;)
+			{
+				x = e->x + i;
+
+				x /= TILE_SIZE;
+
+				tile = mapTileAt(x, y);
+
+				if (tile >= SLOPE_DOWN_START && tile <= SLOPE_DOWN_END)
+				{
+					return FALSE;
+				}
+
+				if (i == e->w)
+				{
+					break;
+				}
+
+				i += TILE_SIZE;
+
+				if (i > e->w)
+				{
+					i = e->w;
+				}
+			}
+		}
+
+		else
+		{
+			for (i=e->w;;)
+			{
+				x = e->x + i;
+
+				x /= TILE_SIZE;
+
+				tile = mapTileAt(x, y);
+
+				if (tile >= SLOPE_UP_START && tile <= SLOPE_UP_END)
+				{
+					return FALSE;
+				}
+
+				if (i == 0)
+				{
+					break;
+				}
+
+				i -= TILE_SIZE;
+
+				if (i < 0)
+				{
+					i = 0;
+				}
+			}
+		}
+	}
+
+	return TRUE;
+}
+
+int onSingleTile(Entity *e)
+{
+	int leftTile, rightTile, midTile, wallLeft, wallRight;
+	int x = e->x + e->w / 2;
+	int y = e->y + e->h - 1;
+
+	x /= TILE_SIZE;
+	y /= TILE_SIZE;
+
+	y++;
+
+	midTile = mapTileAt(x, y);
+
+	leftTile = mapTileAt(x - 1, y);
+
+	rightTile = mapTileAt(x + 1, y);
+
+	wallLeft = mapTileAt(x - 1, y - 1);
+
+	wallRight = mapTileAt(x + 1, y - 1);
+
+	/* On a tile with nothing either side */
+	if ((midTile != BLANK_TILE && midTile < BACKGROUND_TILE_START) && leftTile == BLANK_TILE && rightTile == BLANK_TILE)
+	{
+		return TRUE;
+	}
+
+	/* On a tile with nothing on the left and a wall on the right */
+	if ((midTile != BLANK_TILE && midTile < BACKGROUND_TILE_START) && leftTile == BLANK_TILE && (wallRight != BLANK_TILE && wallRight < BACKGROUND_TILE_START))
+	{
+		return TRUE;
+	}
+
+	/* On a tile with nothing on the right and a wall on the left */
+	if ((midTile != BLANK_TILE && midTile < BACKGROUND_TILE_START) && rightTile == BLANK_TILE && (wallLeft != BLANK_TILE && wallLeft < BACKGROUND_TILE_START))
+	{
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 int isValidOnMap(Entity *e)
 {
 	int i, x1, x2, y1, y2;
 
+	if (e->x < getMapMinX() || e->x + e->w > getMapMaxX() || e->h < 0 || e->y + e->h > getMapMaxY())
+	{
+		return FALSE;
+	}
+
 	i = e->w > TILE_SIZE ? TILE_SIZE : e->w;
 
 	for (;;)
@@ -1195,6 +1386,140 @@ int isValidOnMap(Entity *e)
 	return TRUE;
 }
 
+int getMapFloor(int x, int y)
+{
+	int maxY = getMapMaxY() / TILE_SIZE;
+	int tileID;
+
+	x /= TILE_SIZE;
+	y /= TILE_SIZE;
+
+	tileID = mapTileAt(x, y);
+
+	while (tileID == BLANK_TILE || (tileID >= BACKGROUND_TILE_START && tileID <= BACKGROUND_TILE_END))
+	{
+		y++;
+
+		if (y >= maxY)
+		{
+			break;
+		}
+
+		tileID = mapTileAt(x, y);
+	}
+
+	y *= TILE_SIZE;
+
+	return y;
+}
+
+int getMapCeiling(int x, int y)
+{
+	int tileID;
+
+	x /= TILE_SIZE;
+	y /= TILE_SIZE;
+
+	tileID = mapTileAt(x, y);
+
+	while (tileID == BLANK_TILE || (tileID >= BACKGROUND_TILE_START && tileID <= BACKGROUND_TILE_END))
+	{
+		y--;
+
+		if (y < 0)
+		{
+			break;
+		}
+
+		tileID = mapTileAt(x, y);
+	}
+
+	y = y * TILE_SIZE + TILE_SIZE;
+
+	return y;
+}
+
+int getMapLeft(int x, int y)
+{
+	int tileID;
+
+	x /= TILE_SIZE;
+	y /= TILE_SIZE;
+
+	tileID = mapTileAt(x, y);
+
+	while (tileID == BLANK_TILE || (tileID >= BACKGROUND_TILE_START && tileID <= BACKGROUND_TILE_END))
+	{
+		x--;
+
+		if (x <= 0)
+		{
+			break;
+		}
+
+		tileID = mapTileAt(x, y);
+	}
+
+	x *= TILE_SIZE;
+
+	return x;
+}
+
+int getMapRight(int x, int y)
+{
+	int maxX = getMapMaxX() / TILE_SIZE;
+	int tileID;
+
+	x /= TILE_SIZE;
+	y /= TILE_SIZE;
+
+	tileID = mapTileAt(x, y);
+
+	while (tileID == BLANK_TILE || (tileID >= BACKGROUND_TILE_START && tileID <= BACKGROUND_TILE_END))
+	{
+		x++;
+
+		if (x >= maxX)
+		{
+			break;
+		}
+
+		tileID = mapTileAt(x, y);
+	}
+
+	x *= TILE_SIZE;
+
+	return x;
+}
+
+int getWaterTop(int x, int y)
+{
+	int tileID;
+
+	x /= TILE_SIZE;
+	y /= TILE_SIZE;
+
+	tileID = mapTileAt(x, y);
+
+	while (tileID == WATER_TILE_START)
+	{
+		y--;
+
+		if (y < 0)
+		{
+			break;
+		}
+
+		tileID = mapTileAt(x, y);
+	}
+
+	y++;
+
+	y *= TILE_SIZE;
+
+	return y;
+}
+
 /* Very standard 2D collision detection routine */
 
 int collision(int x0, int y0, int w0, int h0, int x2, int y2, int w1, int h1)
