@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,17 +14,24 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 int collision(int, int, int, int, int, int, int, int);
 void doCollisions(void);
 void checkToMap(Entity *);
 int isAtEdge(Entity *);
+int isAtCeilingEdge(Entity *);
 int isValidOnMap(Entity *);
 Entity *isSpaceEmpty(Entity *);
-void checkEntityToEntity(Entity *);
+Entity *checkEntityToEntity(Entity *);
 void addToGrid(Entity *);
 void initCollisionGrid(void);
 void freeCollisionGrid(void);
-void checkEntityToEntity(Entity *);
+int getMapFloor(int, int);
+int getMapCeiling(int, int);
+int getMapLeft(int, int);
+int getMapRight(int, int);
+int isNearObstacle(Entity *);
+int getWaterTop(int, int);
+int onSingleTile(Entity *);
