@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 void freeGlobalTriggers(void);
@@ -23,3 +23,5 @@ void fireGlobalTrigger(char *);
 void writeGlobalTriggersToFile(FILE *);
 void addGlobalTriggerFromScript(char *);
 void updateGlobalTrigger(char *, int);
+SDL_Surface *listObjectives(void);
+void removeGlobalTrigger(char *);
