@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 void initGame(void);
@@ -33,9 +33,21 @@ void setNextLevelFromScript(char *);
 void writeGameSettingsToFile(FILE *);
 void readGameSettingsFromFile(char *);
 void resetGameSettings(void);
+void initGameOver(void);
 void doGameOver(void);
 void drawGameOver(void);
 void focusLost(void);
 void showSaveDialog(void);
 void pauseGameInventory(void);
 char *getPlayTimeAsString(void);
+void fadeFromColour(int, int, int, int);
+void increaseKillCount(void);
+char *getSlimeTimeAsString(void);
+void increaseSecretsFound(void);
+void setContinuePoint(int, char *, void (*)(void));
+void getContinuePoint(void);
+void clearContinuePoint(void);
+char *getPlayTimeHours(void);
+void drawWeather(void);
+void updateExitCount(int);
+void showEndCredits(void);
