@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,10 +14,13 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 TTF_Font *loadFont(char *, int);
+TTF_Font *loadCustomFont(char *, int);
 void closeFont(TTF_Font *);
 void drawString(char *, int, int, TTF_Font *, int, int, int, int, int);
 SDL_Surface *generateTextSurface(char *, TTF_Font *, int, int, int, int, int, int);
+SDL_Surface *generateTransparentTextSurface(char *, TTF_Font *, int, int, int, int);
+int getCharacterCodeForTestString(void);
