@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,35 +14,35 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <math.h>
-#include <time.h>
 #include <ctype.h>
-#include <locale.h>
-#include <zlib.h>
 #include <dirent.h>
-#include <sys/types.h>
-#include <stdarg.h>
 #include <errno.h>
+#include <inttypes.h>
+#include <locale.h>
+#include <math.h>
+#include <stdio.h>
+#include <stdarg.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
+#include <zlib.h>
 
-#ifndef NO_GETTEXT
-	#include <libintl.h>
-#endif
-
-#ifndef WIN32
-	#include <sys/stat.h>
+#if UNIX == 1
 	#include <pwd.h>
 	#include <unistd.h>
-	#include <errno.h>
 #else
 	#include "strtok_r.h"
 #endif
 
+#ifdef _WIN32
+	#include <windows.h>
+#endif
+
 #include "SDL/SDL.h"
 #include "SDL/SDL_endian.h"
 #include "SDL/SDL_image.h"
@@ -50,3 +50,4 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "SDL/SDL_ttf.h"
 
 #include "structs.h"
+#include "i18n.h"
