@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,14 +14,19 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 void initHud(void);
 void drawHud(void);
 void freeHud(void);
 void doHud(void);
-void setInfoBoxMessage(int, char *, ...);
+void setInfoBoxMessage(int, int, int, int, char *, ...);
 void freeMessageQueue(void);
 void initBossHealthBar(void);
 void freeBossHealthBar(void);
+void drawSpotlight(int, int);
+void showMedal(int, char *);
+int spotlightSize(void);
+SDL_Surface *getMedalImage(int, int);
+void setSlimeTimerValue(int);
