@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,20 +14,24 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 #include "headers.h"
 
-#include "system/record.h"
-#include "init.h"
+#include "credits.h"
 #include "game.h"
+#include "graphics/graphics.h"
+#include "init.h"
+#include "system/record.h"
 
 extern Input input, menuInput;
 extern Game game;
 extern Control control;
 extern Entity player;
 
+static char *getJoystickButton(int);
+
 void getInput(int gameType)
 {
 	int key, button;
@@ -60,17 +64,44 @@ void getInput(int gameType)
 			case SDL_KEYDOWN:
 				key = event.key.keysym.sym;
 
-				if (key == SDLK_ESCAPE || key == control.button[CONTROL_PAUSE])
+				input.lastPressedKey = key;
+
+				if (game.status == IN_ERROR)
+				{
+					if (key == SDLK_ESCAPE)
+					{
+						exit(0);
+					}
+				}
+
+				else if (game.status == IN_TITLE)
+				{
+					pauseGame();
+
+					return;
+				}
+
+				else if (game.status == IN_CREDITS)
+				{
+					if (key == SDLK_ESCAPE || key == control.button[CONTROL_PAUSE] || key == control.button[CONTROL_ATTACK])
+					{
+						fadeCredits();
+					}
+				}
+
+				else if (key == SDLK_ESCAPE || key == control.button[CONTROL_PAUSE])
 				{
 					pauseGame();
 				}
 
-				if (key == control.button[CONTROL_INVENTORY])
+				else if (key == control.button[CONTROL_INVENTORY])
 				{
-					if (!(player.flags & HELPLESS))
+					if (!(player.flags & HELPLESS) && player.action == NULL)
 					{
 						pauseGameInventory();
 					}
+
+					input.inventory = TRUE;
 				}
 
 				else if (key == control.button[CONTROL_LEFT])
@@ -138,11 +169,6 @@ void getInput(int gameType)
 					input.attack = TRUE;
 				}
 
-				else if (key == control.button[CONTROL_FLY])
-				{
-					input.fly = TRUE;
-				}
-
 				else if (key == control.button[CONTROL_SNAP])
 				{
 					input.snap = TRUE;
@@ -166,7 +192,7 @@ void getInput(int gameType)
 
 				else if (key == SCREENSHOT_KEY && game.gameType != RECORDING)
 				{
-					takeSingleScreenshot();
+					takeSingleScreenshot("edgar.bmp");
 				}
 
 				if (key == SDLK_UP)
@@ -263,11 +289,6 @@ void getInput(int gameType)
 					input.attack = FALSE;
 				}
 
-				else if (key == control.button[CONTROL_FLY])
-				{
-					input.fly = FALSE;
-				}
-
 				else if (key == control.button[CONTROL_SNAP])
 				{
 					input.snap = FALSE;
@@ -308,6 +329,11 @@ void getInput(int gameType)
 				{
 					menuInput.attack = FALSE;
 				}
+
+				else if (key == control.button[CONTROL_INVENTORY])
+				{
+					input.inventory = FALSE;
+				}
 			break;
 
 			case SDL_MOUSEBUTTONDOWN:
@@ -345,20 +371,29 @@ void getInput(int gameType)
 			case SDL_JOYBUTTONDOWN:
 				key = event.jbutton.button;
 
-				if (key == control.button[CONTROL_PAUSE])
+				key = -(key + 1000);
+
+				if (game.status == IN_CREDITS)
+				{
+					if (key == control.button[CONTROL_PAUSE] || key == control.button[CONTROL_ATTACK])
+					{
+						fadeCredits();
+					}
+				}
+
+				else if (game.status == IN_TITLE)
+				{
+					pauseGame();
+
+					return;
+				}
+
+				else if (key == control.button[CONTROL_PAUSE])
 				{
 					pauseGame();
 				}
 
-				if (key == control.button[CONTROL_INVENTORY])
-				{
-					if (!(player.flags & HELPLESS))
-					{
-						pauseGameInventory();
-					}
-				}
-
-				if (key == control.button[CONTROL_LEFT])
+				else if (key == control.button[CONTROL_LEFT])
 				{
 					input.left = TRUE;
 				}
@@ -418,11 +453,23 @@ void getInput(int gameType)
 					input.interact = TRUE;
 					input.grabbing = TRUE;
 				}
+
+				else if (key == control.button[CONTROL_INVENTORY])
+				{
+					if (!(player.flags & HELPLESS) && player.action == NULL)
+					{
+						pauseGameInventory();
+					}
+
+					input.inventory = TRUE;
+				}
 			break;
 
 			case SDL_JOYBUTTONUP:
 				key = event.jbutton.button;
 
+				key = -(key + 1000);
+
 				if (key == control.button[CONTROL_LEFT])
 				{
 					input.left = FALSE;
@@ -478,44 +525,71 @@ void getInput(int gameType)
 					input.interact = FALSE;
 					input.grabbing = FALSE;
 				}
+
+				else if (key == control.button[CONTROL_INVENTORY])
+				{
+					input.inventory = FALSE;
+				}
 			break;
 
 			case SDL_JOYAXISMOTION:
 				if (event.jaxis.axis == 0)
 				{
-					if (event.jaxis.value < -3200)
+					if (event.jaxis.value < -control.deadZone)
 					{
-						input.left = TRUE;
+						if (input.xAxisMoved == FALSE || input.right == TRUE)
+						{
+							input.left = TRUE;
+							input.right = FALSE;
+							input.xAxisMoved = TRUE;
+						}
 					}
 
-					else if (event.jaxis.value > 3200)
+					else if (event.jaxis.value > control.deadZone)
 					{
-						input.right = TRUE;
+						if (input.xAxisMoved == FALSE || input.left == TRUE)
+						{
+							input.left = FALSE;
+							input.right = TRUE;
+							input.xAxisMoved = TRUE;
+						}
 					}
 
 					else
 					{
 						input.left = FALSE;
 						input.right = FALSE;
+						input.xAxisMoved = FALSE;
 					}
 				}
 
 				if (event.jaxis.axis == 1)
 				{
-					if (event.jaxis.value < -3200)
+					if (event.jaxis.value < -control.deadZone)
 					{
-						input.up = TRUE;
+						if (input.yAxisMoved == FALSE || input.down == TRUE)
+						{
+							input.up = TRUE;
+							input.down = FALSE;
+							input.yAxisMoved = TRUE;
+						}
 					}
 
-					else if (event.jaxis.value > 3200)
+					else if (event.jaxis.value > control.deadZone)
 					{
-						input.down = TRUE;
+						if (input.yAxisMoved == FALSE || input.up == TRUE)
+						{
+							input.up = FALSE;
+							input.down = TRUE;
+							input.yAxisMoved = TRUE;
+						}
 					}
 
 					else
 					{
 						input.up = FALSE;
 						input.down = FALSE;
+						input.yAxisMoved = FALSE;
 					}
 				}
 			break;
@@ -536,12 +610,22 @@ void getInput(int gameType)
 
 			case REPLAYING:
 				input = getBuffer();
+
+				if (input.inventory == TRUE)
+				{
+					if (!(player.flags & HELPLESS) && player.action == NULL)
+					{
+						pauseGameInventory();
+					}
+				}
 			break;
 
 			default:
 			break;
 		}
 	}
+
+	input.inventory = FALSE;
 }
 
 void resetControls(int editor)
@@ -560,7 +644,6 @@ void resetControls(int editor)
 	control.button[CONTROL_ACTIVATE] = -1;
 	control.button[CONTROL_CUT] = -1;
 	control.button[CONTROL_ATTACK] = -1;
-	control.button[CONTROL_FLY] = -1;
 	control.button[CONTROL_SNAP] = -1;
 	control.button[CONTROL_INTERACT] = -1;
 	control.button[CONTROL_PAUSE] = -1;
@@ -582,7 +665,6 @@ void resetControls(int editor)
 		control.button[CONTROL_ACTIVATE] = 0;
 		control.button[CONTROL_CUT] = -1;
 		control.button[CONTROL_ATTACK] = 2;
-		control.button[CONTROL_FLY] = SDLK_f;
 		control.button[CONTROL_SNAP] = -1;
 		control.button[CONTROL_INTERACT] = 3;
 		control.button[CONTROL_PAUSE] = 9;
@@ -599,21 +681,24 @@ void resetControls(int editor)
 		control.button[CONTROL_ADD] = SDLK_LEFT;
 		control.button[CONTROL_NEXT] = SDLK_PERIOD;
 		control.button[CONTROL_PREVIOUS] = SDLK_COMMA;
-		control.button[CONTROL_SAVE] = SDLK_s;
-		control.button[CONTROL_LOAD] = SDLK_l;
-		control.button[CONTROL_TOGGLE] = SDLK_e;
-		control.button[CONTROL_ACTIVATE] = SDLK_a;
-		control.button[CONTROL_CUT] = SDLK_x;
 		control.button[CONTROL_ATTACK] = SDLK_LCTRL;
 		control.button[CONTROL_INVENTORY] = SDLK_z;
-		#if DEV == 1
-			control.button[CONTROL_FLY] = SDLK_f;
-		#endif
-		control.button[CONTROL_SNAP] = SDLK_g;
+		control.button[CONTROL_ACTIVATE] = SDLK_a;
 		control.button[CONTROL_INTERACT] = SDLK_c;
 		control.button[CONTROL_PAUSE] = SDLK_p;
 		control.button[CONTROL_BLOCK] = SDLK_LALT;
+
+		if (editor == TRUE)
+		{
+			control.button[CONTROL_SAVE] = SDLK_s;
+			control.button[CONTROL_LOAD] = SDLK_l;
+			control.button[CONTROL_TOGGLE] = SDLK_e;
+			control.button[CONTROL_CUT] = SDLK_x;
+			control.button[CONTROL_SNAP] = SDLK_g;
+		}
 	}
+
+	control.deadZone = 6000;
 }
 
 void flushInputs()
@@ -644,6 +729,8 @@ int getSingleInput()
 
 			case SDL_JOYBUTTONDOWN:
 				key = event.jbutton.button;
+
+				key = -(key + 1000);
 			break;
 		}
 	}
@@ -667,12 +754,15 @@ void writeControlsToFile(FILE *fp)
 	fprintf(fp, "ACTIVATE %d\n", control.button[CONTROL_ACTIVATE]);
 	fprintf(fp, "INTERACT %d\n", control.button[CONTROL_INTERACT]);
 	fprintf(fp, "PAUSE %d\n", control.button[CONTROL_PAUSE]);
+	fprintf(fp, "DEAD_ZONE %d\n", control.deadZone);
 }
 
 void readControlsFromFile(char *buffer)
 {
 	char *line, *token, *savePtr;
 
+	savePtr = NULL;
+
 	line = strtok_r(buffer, "\n", &savePtr);
 
 	while (line != NULL)
@@ -770,6 +860,13 @@ void readControlsFromFile(char *buffer)
 			control.button[CONTROL_INVENTORY] = atoi(token);
 		}
 
+		else if (strcmpignorecase(token, "DEAD_ZONE") == 0)
+		{
+			token = strtok(NULL, "\0");
+
+			control.deadZone = atoi(token);
+		}
+
 		else if (strcmpignorecase(token, "GAME_SETTINGS") == 0)
 		{
 			readGameSettingsFromFile(savePtr);
@@ -778,3 +875,36 @@ void readControlsFromFile(char *buffer)
 		line = strtok_r(NULL, "\n", &savePtr);
 	}
 }
+
+char *getKeyValue(int key)
+{
+	char *text;
+
+	if (key == -1)
+	{
+		text = "?";
+	}
+
+	else if (key < 0)
+	{
+		text = getJoystickButton(key);
+	}
+
+	else
+	{
+		text = SDL_GetKeyName(key);
+	}
+
+	return text;
+}
+
+static char *getJoystickButton(int val)
+{
+	static char text[MAX_VALUE_LENGTH];
+
+	val = abs(val) - 1000;
+
+	snprintf(text, MAX_VALUE_LENGTH, _("Joy Button #%d"), val);
+
+	return text;
+}
