@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,21 +14,24 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 #include "headers.h"
 
+#include "dialog.h"
+#include "entity.h"
+#include "event/global_trigger.h"
+#include "event/trigger.h"
 #include "graphics/animation.h"
 #include "graphics/graphics.h"
-#include "player.h"
-#include "item/item.h"
 #include "hud.h"
-#include "event/trigger.h"
-#include "event/global_trigger.h"
+#include "inventory.h"
+#include "item/item.h"
+#include "item/key_items.h"
+#include "player.h"
+#include "system/error.h"
 #include "system/properties.h"
-#include "entity.h"
-#include "dialog.h"
 
 static Inventory inventory;
 extern Entity *self;
@@ -40,7 +43,15 @@ void freeInventory()
 {
 	/* Clear the list */
 
-	memset(&inventory, 0, sizeof(Inventory) * MAX_INVENTORY_ITEMS);
+	memset(inventory.item, 0, sizeof(Entity) * MAX_INVENTORY_ITEMS);
+
+	inventory.selectedIndex = 0;
+
+	inventory.cursorIndex = 0;
+
+	inventory.hasLightningSword = FALSE;
+
+	inventory.hasBow = FALSE;
 
 	if (inventory.background != NULL)
 	{
@@ -62,11 +73,19 @@ void freeInventory()
 
 		inventory.cursor = NULL;
 	}
+
+	if (inventory.objectives != NULL)
+	{
+		SDL_FreeSurface(inventory.objectives);
+
+		inventory.objectives = NULL;
+	}
 }
 
 int addToInventory(Entity *e)
 {
 	int i, found;
+	Entity *temp;
 
 	found = FALSE;
 
@@ -74,11 +93,16 @@ int addToInventory(Entity *e)
 	{
 		for (i=0;i<MAX_INVENTORY_ITEMS;i++)
 		{
-			if (strcmpignorecase(inventory.item[i].objectiveName, e->objectiveName) == 0)
+			if (strcmpignorecase(inventory.item[i].objectiveName, e->objectiveName) == 0 && inventory.item[i].inUse == TRUE)
 			{
 				if (inventory.item[i].health < MAX_STACKABLES)
 				{
-					inventory.item[i].health++;
+					inventory.item[i].health += (e->health == 0 ? 1 : e->health);
+
+					if (inventory.item[i].health > MAX_STACKABLES)
+					{
+						inventory.item[i].health = MAX_STACKABLES;
+					}
 
 					found = TRUE;
 
@@ -87,7 +111,7 @@ int addToInventory(Entity *e)
 
 				else
 				{
-					setInfoBoxMessage(10, _("Cannot carry any more %s"), inventory.item[i].objectiveName);
+					setInfoBoxMessage(0, 255, 255, 255, _("Cannot carry any more %s"), _(inventory.item[i].objectiveName));
 
 					return FALSE;
 				}
@@ -101,21 +125,84 @@ int addToInventory(Entity *e)
 		{
 			if (inventory.item[i].inUse == FALSE)
 			{
+				if (e->action == &generalItemAction)
+				{
+					e->action = &doNothing;
+				}
+
+				if (strcmpignorecase(e->name, "item/safe_combination") == 0)
+				{
+					temp = self;
+
+					self = e;
+
+					self->activate(1);
+
+					self = temp;
+				}
+
+				e->flags &= ~DO_NOT_PERSIST;
+
 				inventory.item[i] = *e;
 
 				inventory.item[i].face = RIGHT;
 
 				inventory.item[i].thinkTime = 0;
 
-				setEntityAnimation(&inventory.item[i], STAND);
+				setEntityAnimationByID(&inventory.item[i], 0);
 
 				if (inventory.item[i].type == WEAPON)
 				{
-					autoSetPlayerWeapon(&inventory.item[i]);
+					if (strcmpignorecase(inventory.item[i].name, "weapon/bow") == 0)
+					{
+						inventory.hasBow = TRUE;
+					}
+
+					else if (strcmpignorecase(inventory.item[i].name, "weapon/lightning_sword") == 0)
+					{
+						inventory.hasLightningSword = TRUE;
+					}
+
+					else if (strcmpignorecase(inventory.item[i].name, "weapon/lightning_sword_empty") == 0)
+					{
+						inventory.hasLightningSword = TRUE;
+					}
+
+					if (strcmpignorecase(inventory.item[i].name, "weapon/normal_arrow") != 0 &&
+						strcmpignorecase(inventory.item[i].name, "weapon/flaming_arrow") != 0)
+					{
+						autoSetPlayerWeapon(&inventory.item[i]);
+					}
 				}
 
 				else if (inventory.item[i].type == SHIELD)
 				{
+					if (strcmpignorecase(inventory.item[i].name, "weapon/disintegration_shield") == 0)
+					{
+						switch (inventory.item[i].health)
+						{
+							case 1:
+								loadProperties("weapon/disintegration_shield_1", &inventory.item[i]);
+							break;
+
+							case 2:
+								loadProperties("weapon/disintegration_shield_2", &inventory.item[i]);
+							break;
+
+							case 3:
+								loadProperties("weapon/disintegration_shield_3", &inventory.item[i]);
+							break;
+
+							case 4:
+								loadProperties("weapon/disintegration_shield_4", &inventory.item[i]);
+							break;
+
+							default:
+								loadProperties("weapon/disintegration_shield", &inventory.item[i]);
+							break;
+						}
+					}
+
 					autoSetPlayerShield(&inventory.item[i]);
 				}
 
@@ -133,15 +220,30 @@ int addToInventory(Entity *e)
 
 	if (found == TRUE)
 	{
-		e->inUse = FALSE;
+		if ((inventory.item[i].flags & STACKABLE) && e->health > 1)
+		{
+			setInfoBoxMessage(60, 255, 255, 255, _("Picked up %s x %d"), _(inventory.item[i].objectiveName), e->health);
+		}
 
-		setInfoBoxMessage(60,  _("Picked up %s"), inventory.item[i].objectiveName);
+		else
+		{
+			setInfoBoxMessage(60, 255, 255, 255, _("Picked up %s"), _(inventory.item[i].objectiveName));
+		}
+
+		e->inUse = FALSE;
 
 		fireTrigger(inventory.item[i].objectiveName);
 
 		fireGlobalTrigger(inventory.item[i].objectiveName);
 	}
 
+	else
+	{
+		setInfoBoxMessage(0, 255, 255, 255, _("Cannot pick up %s. Inventory full"), _(e->objectiveName));
+
+		return FALSE;
+	}
+
 	return found;
 }
 
@@ -149,14 +251,14 @@ void moveInventoryCursor(int index)
 {
 	inventory.cursorIndex += index;
 
-	if (index == 1 && (inventory.cursorIndex % INVENTORY_BOX_COUNT) == 0)
+	if (index == 1 && (inventory.cursorIndex % INVENTORY_COLUMN_COUNT) == 0)
 	{
-		inventory.cursorIndex -= INVENTORY_BOX_COUNT;
+		inventory.cursorIndex -= INVENTORY_COLUMN_COUNT;
 	}
 
-	else if (index == -1 && (inventory.cursorIndex == -1 || (inventory.cursorIndex % INVENTORY_BOX_COUNT) == INVENTORY_BOX_COUNT - 1))
+	else if (index == -1 && (inventory.cursorIndex == -1 || (inventory.cursorIndex % INVENTORY_COLUMN_COUNT) == INVENTORY_COLUMN_COUNT - 1))
 	{
-		inventory.cursorIndex += INVENTORY_BOX_COUNT;
+		inventory.cursorIndex += INVENTORY_COLUMN_COUNT;
 	}
 
 	if (inventory.cursorIndex >= MAX_INVENTORY_ITEMS)
@@ -203,6 +305,8 @@ void nextInventoryItem(int index)
 		}
 	}
 
+	inventory.cursorIndex = inventory.selectedIndex;
+
 	if (inventory.description != NULL)
 	{
 		SDL_FreeSurface(inventory.description);
@@ -211,15 +315,57 @@ void nextInventoryItem(int index)
 	}
 }
 
-void selectInventoryItem(void)
+void moveInventoryItem(int index)
+{
+	Entity temp;
+
+	if (inventory.item[inventory.cursorIndex].inUse == TRUE)
+	{
+		if (index == 1 && inventory.item[inventory.cursorIndex + index].inUse == TRUE && inventory.cursorIndex + index < MAX_INVENTORY_ITEMS)
+		{
+			temp = inventory.item[inventory.cursorIndex + index];
+
+			inventory.item[inventory.cursorIndex + index] = inventory.item[inventory.cursorIndex];
+
+			inventory.item[inventory.cursorIndex] = temp;
+
+			moveInventoryCursor(index);
+
+			if (inventory.cursorIndex % INVENTORY_COLUMN_COUNT == 0)
+			{
+				moveInventoryCursor(INVENTORY_COLUMN_COUNT);
+			}
+		}
+
+		else if (index == -1 && inventory.cursorIndex + index >= 0)
+		{
+			temp = inventory.item[inventory.cursorIndex + index];
+
+			inventory.item[inventory.cursorIndex + index] = inventory.item[inventory.cursorIndex];
+
+			inventory.item[inventory.cursorIndex] = temp;
+
+			moveInventoryCursor(index);
+
+			if (inventory.cursorIndex != 0 && (inventory.cursorIndex % INVENTORY_COLUMN_COUNT) == INVENTORY_COLUMN_COUNT - 1)
+			{
+				moveInventoryCursor(-INVENTORY_COLUMN_COUNT);
+			}
+		}
+	}
+}
+
+void selectInventoryItem()
 {
 	if (inventory.item[inventory.cursorIndex].inUse == TRUE)
 	{
 		inventory.selectedIndex = inventory.cursorIndex;
+
+		setInventoryDialogMessage(_("Selected %s"), _(inventory.item[inventory.cursorIndex].objectiveName));
 	}
 }
 
-Entity *getInventoryItem(char *name)
+Entity *getInventoryItemByObjectiveName(char *name)
 {
 	int i;
 
@@ -234,9 +380,78 @@ Entity *getInventoryItem(char *name)
 	return NULL;
 }
 
-int removeInventoryItem(char *name)
+Entity *getInventoryItemByName(char *name)
+{
+	int i;
+
+	for (i=0;i<MAX_INVENTORY_ITEMS;i++)
+	{
+		if (inventory.item[i].inUse == TRUE && strcmpignorecase(inventory.item[i].name, name) == 0)
+		{
+			return &inventory.item[i];
+		}
+	}
+
+	return NULL;
+}
+
+Entity *getCurrentInventoryItem()
+{
+	return inventory.item[inventory.selectedIndex].inUse == TRUE ? &inventory.item[inventory.selectedIndex] : NULL;
+}
+
+void replaceInventoryItem(char *name, Entity *e)
+{
+	int i;
+
+	for (i=0;i<MAX_INVENTORY_ITEMS;i++)
+	{
+		if (inventory.item[i].inUse == TRUE && strcmpignorecase(inventory.item[i].objectiveName, name) == 0)
+		{
+			inventory.item[i] = *e;
+
+			inventory.item[i].face = RIGHT;
+
+			inventory.item[i].thinkTime = 0;
+
+			setEntityAnimationByID(&inventory.item[i], 0);
+
+			return;
+		}
+	}
+
+	showErrorAndExit("Could not find inventory item %s to replace", name);
+}
+
+void replaceInventoryItemWithName(char *name, Entity *e)
+{
+	int i;
+
+	for (i=0;i<MAX_INVENTORY_ITEMS;i++)
+	{
+		if (inventory.item[i].inUse == TRUE && strcmpignorecase(inventory.item[i].name, name) == 0)
+		{
+			inventory.item[i] = *e;
+
+			inventory.item[i].face = RIGHT;
+
+			inventory.item[i].thinkTime = 0;
+
+			inventory.item[i].alpha = 255;
+
+			setEntityAnimationByID(&inventory.item[i], 0);
+
+			return;
+		}
+	}
+
+	showErrorAndExit("Could not find inventory item %s to replace", name);
+}
+
+int removeInventoryItemByObjectiveName(char *name)
 {
 	int i, found;
+	Entity *e;
 
 	found = FALSE;
 
@@ -244,7 +459,34 @@ int removeInventoryItem(char *name)
 	{
 		if (inventory.item[i].inUse == TRUE && strcmpignorecase(inventory.item[i].objectiveName, name) == 0)
 		{
-			inventory.item[i].inUse = FALSE;
+			if (inventory.item[i].flags & STACKABLE)
+			{
+				inventory.item[i].health--;
+
+				if (inventory.item[i].health <= 0)
+				{
+					inventory.item[i].inUse = FALSE;
+				}
+			}
+
+			else
+			{
+				inventory.item[i].inUse = FALSE;
+
+				if (strcmpignorecase(playerWeapon.name, inventory.item[i].name) == 0)
+				{
+					e = removePlayerWeapon();
+
+					e->inUse = FALSE;
+				}
+
+				else if (strcmpignorecase(playerShield.name, inventory.item[i].name) == 0)
+				{
+					e = removePlayerShield();
+
+					e->inUse = FALSE;
+				}
+			}
 
 			found = TRUE;
 
@@ -260,16 +502,45 @@ int removeInventoryItem(char *name)
 	return found;
 }
 
-void dropInventoryItem()
+int removeInventoryItemByName(char *name)
 {
-	if (inventory.item[inventory.selectedIndex].inUse == TRUE)
+	int i, found;
+	Entity *e;
+
+	found = FALSE;
+
+	for (i=0;i<MAX_INVENTORY_ITEMS;i++)
 	{
-		dropItem(&inventory.item[inventory.selectedIndex]);
+		if (inventory.item[i].inUse == TRUE && strcmpignorecase(inventory.item[i].name, name) == 0)
+		{
+			inventory.item[i].inUse = FALSE;
 
-		inventory.item[inventory.selectedIndex].inUse = FALSE;
+			if (strcmpignorecase(playerWeapon.name, inventory.item[i].name) == 0)
+			{
+				e = removePlayerWeapon();
 
+				e->inUse = FALSE;
+			}
+
+			else if (strcmpignorecase(playerShield.name, inventory.item[i].name) == 0)
+			{
+				e = removePlayerShield();
+
+				e->inUse = FALSE;
+			}
+
+			found = TRUE;
+
+			break;
+		}
+	}
+
+	if (found == TRUE)
+	{
 		sortInventory();
 	}
+
+	return found;
 }
 
 void useInventoryItem()
@@ -370,7 +641,7 @@ void addRequiredToInventory(Entity *other)
 
 	if (!(self->flags & INVULNERABLE) && other->type == PLAYER)
 	{
-		item = getInventoryItem(self->requires);
+		item = getInventoryItemByObjectiveName(self->requires);
 
 		if (item != NULL)
 		{
@@ -378,7 +649,7 @@ void addRequiredToInventory(Entity *other)
 
 			self->inUse = FALSE;
 
-			setInfoBoxMessage(60,  _("Picked up %s"), self->objectiveName);
+			setInfoBoxMessage(60, 255, 255, 255, _("Picked up %s"), _(self->objectiveName));
 
 			fireTrigger(self->objectiveName);
 
@@ -387,7 +658,7 @@ void addRequiredToInventory(Entity *other)
 
 		else
 		{
-			setInfoBoxMessage(60, _("%s is required to carry this item"), self->requires);
+			setInfoBoxMessage(60, 255, 255, 255, _("%s is required to carry this item"), _(self->requires));
 		}
 	}
 }
@@ -403,12 +674,16 @@ void loadInventoryItems()
 		{
 			loadProperties(inventory.item[i].name, &e);
 
+			inventory.item[i].fallout = &keyItemFallout;
+
 			for (j=0;j<MAX_ANIMATION_TYPES;j++)
 			{
 				inventory.item[i].animation[j] = e.animation[j];
 			}
 
-			setEntityAnimation(&inventory.item[i], STAND);
+			inventory.item[i].currentAnim = -1;
+
+			setEntityAnimationByID(&inventory.item[i], 0);
 
 			if (inventory.item[i].type == WEAPON && strcmpignorecase(inventory.item[i].name, playerWeapon.name) == 0)
 			{
@@ -423,44 +698,61 @@ void loadInventoryItems()
 
 				setPlayerShield(1);
 			}
+
+			else if (inventory.item[i].type == TEMP_ITEM)
+			{
+				inventory.item[i].type = ITEM;
+
+				inventory.item[i].action = &doNothing;
+
+				inventory.item[i].flags &= ~DO_NOT_PERSIST;
+			}
 		}
 	}
-
-	printf("Done reloading inventory\n");
 }
 
 void getInventoryItemFromScript(char *line)
 {
-	char command[15], itemName[MAX_VALUE_LENGTH], entityName[MAX_VALUE_LENGTH];
-	int quantity, success, failure, quantityToRemove;
+	char command[15], itemName[MAX_VALUE_LENGTH], entityName[MAX_VALUE_LENGTH], quiet[MAX_VALUE_LENGTH];
+	int quantity, success, failure, quantityToRemove, read;
 	Entity *e, *item;
 
-	sscanf(line, "%s \"%[^\"]\" %d %d %s %d %d", command, itemName, &quantity, &quantityToRemove, entityName, &success, &failure);
+	read = sscanf(line, "%s \"%[^\"]\" %d %d %s %d %d %s", command, itemName, &quantity, &quantityToRemove, entityName, &success, &failure, quiet);
+
+	if (read < 7)
+	{
+		showErrorAndExit("HAS_ITEM or REMOVE command has wrong number of arguments");
+	}
 
 	e = getEntityByObjectiveName(entityName);
 
 	if (e == NULL)
 	{
-		printf("Could not find Entity %s to give item %s to\n", entityName, itemName);
-
-		exit(1);
+		showErrorAndExit("Could not find Entity %s to give item %s to", entityName, itemName);
 	}
 
-	item = getInventoryItem(itemName);
+	item = getInventoryItemByObjectiveName(itemName);
 
 	if (item != NULL && (item->health >= quantity || quantity == 1))
 	{
 		if (strcmpignorecase(command, "REMOVE") == 0)
 		{
 			item->health -= quantityToRemove;
-			
+
 			updateTrigger(itemName, quantityToRemove);
-			
+
 			updateGlobalTrigger(itemName, quantityToRemove);
 
 			if (item->health <= 0 || quantityToRemove == -1)
 			{
-				removeInventoryItem(itemName);
+				item->health = 0;
+
+				removeInventoryItemByObjectiveName(itemName);
+
+				if (read == 7)
+				{
+					setInfoBoxMessage(90, 255, 255, 255, _("Removed %s"), _(itemName));
+				}
 			}
 		}
 
@@ -469,20 +761,30 @@ void getInventoryItemFromScript(char *line)
 
 	else
 	{
-		if (item == NULL)
-		{
-			printf("Could not find item %s\n", itemName);
-		}
-
-		else
-		{
-			printf("Only found %d of %d required\n", item->health, quantity);
-		}
-
 		e->health = failure;
 	}
 }
 
+void useInventoryItemFromScript(char *itemName)
+{
+	Entity *item, *temp;
+
+	item = getInventoryItemByObjectiveName(itemName);
+
+	if (item == NULL)
+	{
+		showErrorAndExit("Could not find Inventory Item %s", itemName);
+	}
+
+	temp = self;
+
+	self = item;
+
+	self->activate(0);
+
+	self = temp;
+}
+
 void writeInventoryToFile(FILE *fp)
 {
 	int i;
@@ -501,9 +803,10 @@ void writeInventoryToFile(FILE *fp)
 			fprintf(fp, "END_X %d\n", (int)self->endX);
 			fprintf(fp, "END_Y %d\n", (int)self->endY);
 			fprintf(fp, "THINKTIME %d\n", self->thinkTime);
+			fprintf(fp, "MENTAL %d\n", self->mental);
 			fprintf(fp, "HEALTH %d\n", self->health);
 			fprintf(fp, "DAMAGE %d\n", self->damage);
-			fprintf(fp, "SPEED %0.1f\n", self->speed);
+			fprintf(fp, "SPEED %0.2f\n", self->speed);
 			fprintf(fp, "WEIGHT %0.2f\n", self->weight);
 			fprintf(fp, "OBJECTIVE_NAME %s\n", self->objectiveName);
 			fprintf(fp, "REQUIRES %s\n", self->requires);
@@ -514,23 +817,38 @@ void writeInventoryToFile(FILE *fp)
 	}
 }
 
+void clearInventoryDescription()
+{
+	if (inventory.description != NULL)
+	{
+		SDL_FreeSurface(inventory.description);
+
+		inventory.description = NULL;
+	}
+
+	if (inventory.objectives != NULL)
+	{
+		SDL_FreeSurface(inventory.objectives);
+
+		inventory.objectives = NULL;
+	}
+}
+
 void drawInventory()
 {
-	int i, x, y;
+	int i, x, y, descriptionY;
 	char description[MAX_MESSAGE_LENGTH];
 	Entity *e;
 
 	if (inventory.background == NULL)
 	{
-		inventory.background = createSurface(INVENTORY_BOX_SIZE * INVENTORY_BOX_COUNT, INVENTORY_BOX_SIZE * INVENTORY_BOX_COUNT);
-
-		SDL_SetAlpha(inventory.background, SDL_SRCALPHA|SDL_RLEACCEL, 196);
+		inventory.background = createSurface(INVENTORY_BOX_SIZE * INVENTORY_COLUMN_COUNT, INVENTORY_BOX_SIZE * INVENTORY_ROW_COUNT);
 
 		inventory.x = (SCREEN_WIDTH - inventory.background->w) / 2;
-		inventory.y = (SCREEN_HEIGHT - inventory.background->h) / 2;
+		inventory.y = 64;
 	}
 
-	drawImage(inventory.background, inventory.x, inventory.y, FALSE);
+	drawImage(inventory.background, inventory.x, inventory.y, FALSE, 196);
 
 	x = inventory.x;
 	y = inventory.y;
@@ -541,12 +859,20 @@ void drawInventory()
 		{
 			e = &inventory.item[i];
 
-			drawLoopingAnimation(e, x, y, INVENTORY_BOX_SIZE, INVENTORY_BOX_SIZE, 1);
+			if (inventory.selectedIndex == i)
+			{
+				drawSprite(e, x, y, INVENTORY_BOX_SIZE, INVENTORY_BOX_SIZE, 1);
+			}
+
+			else
+			{
+				drawLoopingAnimation(e, x, y, INVENTORY_BOX_SIZE, INVENTORY_BOX_SIZE, 1);
+			}
 		}
 
 		x += INVENTORY_BOX_SIZE;
 
-		if (i != 0 && (i % (INVENTORY_BOX_COUNT - 1) == 0))
+		if (i != 0 && ((i + 1) % INVENTORY_COLUMN_COUNT == 0))
 		{
 			y += INVENTORY_BOX_SIZE;
 
@@ -556,34 +882,51 @@ void drawInventory()
 
 	e = &inventory.item[inventory.cursorIndex];
 
+	descriptionY = inventory.y + inventory.background->h + 10;
+
 	if (inventory.description == NULL)
 	{
 		if (e->inUse == TRUE)
 		{
 			if (e->flags & STACKABLE)
 			{
-				snprintf(description, MAX_MESSAGE_LENGTH, _("%s (%d)"), e->description, e->health);
+				snprintf(description, MAX_MESSAGE_LENGTH, "%s (%d)", _(e->description), e->health);
+			}
+
+			else if (strcmpignorecase(e->name, "weapon/lightning_sword") == 0)
+			{
+				snprintf(description, MAX_MESSAGE_LENGTH, "%s (%d)", _(e->objectiveName), e->mental);
 			}
 
 			else if (strlen(e->description) == 0 && strlen(e->objectiveName) != 0)
 			{
-				snprintf(description, MAX_MESSAGE_LENGTH, _("%s"), e->objectiveName);
+				snprintf(description, MAX_MESSAGE_LENGTH, "%s", _(e->objectiveName));
 			}
 
 			else
 			{
-				snprintf(description, MAX_MESSAGE_LENGTH, _("%s"), e->description);
+				snprintf(description, MAX_MESSAGE_LENGTH, "%s", _(e->description));
 			}
 
 			inventory.description = createDialogBox(NULL, description);
 
-			drawImage(inventory.description, (SCREEN_WIDTH - inventory.description->w) / 2, inventory.y + inventory.background->h + 10, FALSE);
+			drawImage(inventory.description, (SCREEN_WIDTH - inventory.description->w) / 2, descriptionY, FALSE, 255);
 		}
 	}
 
 	else
 	{
-		drawImage(inventory.description, (SCREEN_WIDTH - inventory.description->w) / 2, inventory.y + inventory.background->h + 10, FALSE);
+		drawImage(inventory.description, (SCREEN_WIDTH - inventory.description->w) / 2, descriptionY, FALSE, 255);
+	}
+
+	if (inventory.description != NULL)
+	{
+		descriptionY += inventory.description->h + 10;
+	}
+
+	if (inventory.objectives == NULL)
+	{
+		inventory.objectives = listObjectives();
 	}
 
 	if (inventory.cursor == NULL)
@@ -591,13 +934,15 @@ void drawInventory()
 		inventory.cursor = loadImage("gfx/hud/inventory_cursor.png");
 	}
 
-	x = (inventory.cursorIndex % INVENTORY_BOX_COUNT) * INVENTORY_BOX_SIZE;
-	y = (inventory.cursorIndex / INVENTORY_BOX_COUNT) * INVENTORY_BOX_SIZE;
+	x = (inventory.cursorIndex % INVENTORY_COLUMN_COUNT) * INVENTORY_BOX_SIZE;
+	y = (inventory.cursorIndex / INVENTORY_COLUMN_COUNT) * INVENTORY_BOX_SIZE;
 
 	x += inventory.x;
 	y += inventory.y;
 
-	drawImage(inventory.cursor, x, y, FALSE);
+	drawImage(inventory.cursor, x, y, FALSE, 255);
+
+	drawImage(inventory.objectives, (SCREEN_WIDTH - inventory.objectives->w) / 2, descriptionY, FALSE, 255);
 }
 
 void setInventoryDialogMessage(char *fmt, ...)
@@ -616,3 +961,71 @@ void setInventoryDialogMessage(char *fmt, ...)
 
 	inventory.description = createDialogBox(NULL, text);
 }
+
+void setInventoryIndex(int val)
+{
+	inventory.selectedIndex = val;
+
+	moveInventoryCursor(val);
+}
+
+int getInventoryIndex()
+{
+	return inventory.selectedIndex;
+}
+
+int hasBow()
+{
+	return inventory.hasBow;
+}
+
+int hasLightningSword()
+{
+	return inventory.hasLightningSword;
+}
+
+void scriptAddToInventory(char *name, int quiet)
+{
+	int i;
+	Entity *e;
+
+	e = addPermanentItem(name, 0, 0);
+
+	for (i=0;i<MAX_INVENTORY_ITEMS;i++)
+	{
+		if (inventory.item[i].inUse == FALSE)
+		{
+			inventory.item[i] = *e;
+
+			inventory.item[i].face = RIGHT;
+
+			inventory.item[i].thinkTime = 0;
+
+			setEntityAnimationByID(&inventory.item[i], 0);
+
+			if (quiet == FALSE)
+			{
+				setInfoBoxMessage(60, 255, 255, 255, _("Picked up %s"), _(inventory.item[i].objectiveName));
+			}
+
+			break;
+		}
+	}
+
+	e->inUse = FALSE;
+}
+
+Entity *removeInventoryItemAtCursor()
+{
+	sortInventory();
+
+	return (inventory.item[inventory.selectedIndex].inUse == TRUE ? &inventory.item[inventory.selectedIndex] : NULL);
+}
+
+void resetInventoryIndex()
+{
+	inventory.selectedIndex = 0;
+
+	inventory.cursorIndex = 0;
+}
+
