@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,18 +14,17 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 int addToInventory(Entity *);
 void moveInventoryCursor(int);
 void selectInventoryItem(void);
-Entity *getInventoryItem(char *);
-void dropInventoryItem(void);
+Entity *getInventoryItemByObjectiveName(char *);
 void useInventoryItem(void);
 void doInventory(void);
 void drawSelectedInventoryItem(int, int, int, int);
-int removeInventoryItem(char *);
+int removeInventoryItemByObjectiveName(char *);
 void addRequiredToInventory(Entity *);
 void loadInventoryItems(void);
 void writeInventoryToFile(FILE *);
@@ -34,3 +33,18 @@ void freeInventory(void);
 void drawInventory(void);
 void nextInventoryItem(int);
 void setInventoryDialogMessage(char *, ...);
+void clearInventoryDescription(void);
+void useInventoryItemFromScript(char *);
+void replaceInventoryItem(char *, Entity *);
+void replaceInventoryItemWithName(char *, Entity *);
+Entity *getCurrentInventoryItem(void);
+void setInventoryIndex(int);
+int getInventoryIndex(void);
+int hasBow(void);
+int hasLightningSword(void);
+void scriptAddToInventory(char *, int);
+void moveInventoryItem(int);
+Entity *removeInventoryItemAtCursor(void);
+void resetInventoryIndex(void);
+Entity *getInventoryItemByName(char *);
+int removeInventoryItemByName(char *);
