@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 Entity *addPermanentItem(char *, int, int);
@@ -22,5 +22,7 @@ Entity *addTemporaryItem(char *, int, int, int, float, float);
 void dropRandomItem(int, int);
 void generalItemAction(void);
 void healthTouch(Entity *);
-void dropItem(Entity *);
+void lightningChargeTouch(Entity *);
 void throwItem(int);
+Entity *dropCollectableItem(char *, int, int, int);
+void itemFallout(void);
