@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,26 +14,27 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 #include "headers.h"
 
-#include "map.h"
-#include "system/resources.h"
-#include "system/random.h"
-#include "graphics/graphics.h"
 #include "audio/audio.h"
-#include "system/properties.h"
-#include "entity.h"
-#include "world/target.h"
-#include "player.h"
 #include "audio/music.h"
-#include "game.h"
+#include "credits.h"
+#include "entity.h"
 #include "event/trigger.h"
-#include "weather.h"
-#include "system/pak.h"
+#include "game.h"
 #include "geometry.h"
+#include "graphics/graphics.h"
+#include "map.h"
+#include "player.h"
+#include "system/error.h"
+#include "system/pak.h"
+#include "system/random.h"
+#include "system/resources.h"
+#include "weather.h"
+#include "world/target.h"
 
 static Map map;
 static SDL_Surface *mapImages[MAX_TILES];
@@ -50,10 +51,13 @@ static char *extensions[] = {"ogg", "mp3", "wav", NULL};
 
 void loadMap(char *name, int loadEntityResources)
 {
-	int x, y;
+	int x, y, animTileID;
 	char itemName[MAX_MESSAGE_LENGTH], filename[MAX_LINE_LENGTH], *line, *token, *savePtr1, *savePtr2;
 	unsigned char *buffer;
 
+	savePtr1 = NULL;
+	savePtr2 = NULL;
+
 	lavaTile = LAVA_TILE_START;
 
 	slimeTile = SLIME_TILE_START;
@@ -79,10 +83,32 @@ void loadMap(char *name, int loadEntityResources)
 
 	map.maxX = map.maxY = 0;
 
+	map.forceMinY = FALSE;
+
+	/* Reset the clipping */
+
+	map.darkMap = FALSE;
+
 	/* Read the data from the file into the map */
 
 	line = strtok_r((char *)buffer, "\n", &savePtr1);
 
+	/* Reset the animated tiles */
+
+	if (map.animTileTotal > 0)
+	{
+		for (x=0;x<map.animTileTotal;x++)
+		{
+			free(map.animTile[x].tile);
+		}
+
+		free(map.animTile);
+	}
+
+	map.animTileTotal = 0;
+
+	animTileID = 0;
+
 	while (line != NULL)
 	{
 		sscanf(line, "%s", itemName);
@@ -92,8 +118,6 @@ void loadMap(char *name, int loadEntityResources)
 			sscanf(line, "%*s %[^$]s\n", itemName);
 
 			STRNCPY(map.mapName, itemName, sizeof(map.mapName));
-
-			printf("Setting mapName to %s\n", map.mapName);
 		}
 
 		else if (strcmpignorecase(itemName, "TILESET") == 0)
@@ -102,8 +126,6 @@ void loadMap(char *name, int loadEntityResources)
 
 			sscanf(line, "%*s %s\n", itemName);
 
-			printf("Loading tiles from %s\n", itemName);
-
 			loadMapTiles(itemName);
 
 			STRNCPY(map.tilesetName, itemName, sizeof(map.tilesetName));
@@ -111,18 +133,16 @@ void loadMap(char *name, int loadEntityResources)
 
 		else if (strcmpignorecase(itemName, "BACKGROUND_SPEED") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the background speed */
 
 			sscanf(line, "%*s %s\n", itemName);
 
-			printf("Setting background speed to %s\n", itemName);
-
 			map.backgroundSpeed[0] = atof(itemName);
 		}
 
 		else if (strcmpignorecase(itemName, "WRAP_X") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the wrapping */
 
 			sscanf(line, "%*s %s\n", itemName);
 
@@ -131,7 +151,7 @@ void loadMap(char *name, int loadEntityResources)
 
 		else if (strcmpignorecase(itemName, "WRAP_Y") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the wrapping */
 
 			sscanf(line, "%*s %s\n", itemName);
 
@@ -140,18 +160,16 @@ void loadMap(char *name, int loadEntityResources)
 
 		else if (strcmpignorecase(itemName, "BACKGROUND_SPEED_2") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the background speed */
 
 			sscanf(line, "%*s %s\n", itemName);
 
-			printf("Setting background speed 2 to %s\n", itemName);
-
 			map.backgroundSpeed[1] = atof(itemName);
 		}
 
 		else if (strcmpignorecase(itemName, "WRAP_X_2") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the wrapping */
 
 			sscanf(line, "%*s %s\n", itemName);
 
@@ -160,20 +178,27 @@ void loadMap(char *name, int loadEntityResources)
 
 		else if (strcmpignorecase(itemName, "WRAP_Y_2") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the wrapping */
 
 			sscanf(line, "%*s %s\n", itemName);
 
 			map.wrapY[1] = (strcmpignorecase(itemName, "TRUE") == 0 ? TRUE : FALSE);
 		}
 
-		else if (strcmpignorecase(itemName, "AMBIENCE") == 0)
+		else if (strcmpignorecase(itemName, "DARK_MAP") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the darkness */
 
 			sscanf(line, "%*s %s\n", itemName);
 
-			printf("Loading ambience from %s\n", itemName);
+			map.darkMap = (strcmpignorecase(itemName, "TRUE") == 0 ? TRUE : FALSE);
+		}
+
+		else if (strcmpignorecase(itemName, "AMBIENCE") == 0)
+		{
+			/* Load the ambience */
+
+			sscanf(line, "%*s %s\n", itemName);
 
 			loadAmbience(itemName);
 
@@ -182,23 +207,19 @@ void loadMap(char *name, int loadEntityResources)
 
 		else if (strcmpignorecase(itemName, "MUSIC") == 0)
 		{
-			/* Load the map tiles */
+			/* Load the music */
 
 			x = sscanf(line, "%*s %s\n", itemName);
 
 			if (x > 0)
 			{
-				printf("Loading music from %s\n", itemName);
-
-				loadMusic(itemName);
-
 				STRNCPY(map.musicName, itemName, sizeof(map.musicName));
 			}
 		}
 
 		else if (strcmpignorecase(itemName, "WEATHER") == 0)
 		{
-			/* Load the map tiles */
+			/* Set the weather */
 
 			sscanf(line, "%*s %s\n", itemName);
 
@@ -207,6 +228,68 @@ void loadMap(char *name, int loadEntityResources)
 			setWeather(x);
 		}
 
+		else if (strcmpignorecase(itemName, "MIN_Y") == 0)
+		{
+			/* The map's minimum Y */
+
+			sscanf(line, "%*s %s\n", itemName);
+
+			map.forceMinY = (strcmpignorecase(itemName, "TRUE") == 0 ? TRUE : FALSE);
+
+			if (map.forceMinY == TRUE)
+			{
+				map.minY = MAX_MAP_Y;
+			}
+		}
+
+		else if (strcmpignorecase(itemName, "ANIM_TILE_COUNT") == 0)
+		{
+			/* The map's total number of animated tiles */
+
+			sscanf(line, "%*s %s\n", itemName);
+
+			map.animTileTotal = atoi(itemName);
+
+			if (map.animTileTotal > 0)
+			{
+				map.animTile = malloc(sizeof(AnimTile) * map.animTileTotal);
+
+				if (map.animTile == NULL)
+				{
+					showErrorAndExit("Failed to allocate a whole %d bytes for the animated tile count", (int)(sizeof(int *) * map.animTileTotal));
+				}
+			}
+
+			/* A set of animated tiles */
+
+			for (animTileID=0;animTileID<map.animTileTotal;animTileID++)
+			{
+				line = strtok_r(NULL, "\n", &savePtr1);
+
+				y = countTokens(line, " ");
+
+				map.animTile[animTileID].tileCount = y;
+
+				map.animTile[animTileID].tileIndex = 0;
+
+				map.animTile[animTileID].tile = malloc(sizeof(int) * y);
+
+				if (map.animTile[animTileID].tile == NULL)
+				{
+					showErrorAndExit("Failed to allocate a whole %d bytes for the animated tiles", (int)(sizeof(int) * y));
+				}
+
+				token = strtok_r(line, " ", &savePtr2);
+
+				for (x=0;x<y;x++)
+				{
+					map.animTile[animTileID].tile[x] = atoi(token);
+
+					token = strtok_r(NULL, " ", &savePtr2);
+				}
+			}
+		}
+
 		else if (strcmpignorecase(itemName, "DATA") == 0)
 		{
 			line = strtok_r(NULL, "\n", &savePtr1);
@@ -221,9 +304,17 @@ void loadMap(char *name, int loadEntityResources)
 				{
 					map.tile[y][x] = atoi(token);
 					/*
-					if (map.tile[y][x] != 0 && map.tile[y][x] <= 4)
+					if (map.tile[y][x] != 0 && map.tile[y][x] <= 3)
 					{
-						map.tile[y][x] = 1 + (prand() % 4);
+						map.tile[y][x] = 1 + (prand() % 3);
+					}
+
+					if (map.tile[y][x] == 132)
+					{
+						if (prand() % 50 == 0)
+						{
+							map.tile[y][x] = 123 + (prand() % 2);
+						}
 					}
 					*/
 					if (map.tile[y][x] > 0)
@@ -237,14 +328,16 @@ void loadMap(char *name, int loadEntityResources)
 						{
 							map.maxY = y;
 						}
-					}
 
-					if (map.tile[y][x] > 0)
-					{
 						if (x < map.minX)
 						{
 							map.minX = x;
 						}
+
+						if (map.forceMinY == TRUE && y < map.minY)
+						{
+							map.minY = y;
+						}
 					}
 
 					token = strtok_r(NULL, " ", &savePtr2);
@@ -255,8 +348,20 @@ void loadMap(char *name, int loadEntityResources)
 					line = strtok_r(NULL, "\n", &savePtr1);
 				}
 			}
+			/*
+			for (y=0;y<MAX_MAP_Y;y++)
+			{
+				for (x=0;x<MAX_MAP_X;x++)
+				{
+					if (map.tile[y][x] == 4 && map.tile[y + 1][x] == 0)
+					{
+						map.tile[y][x] = 7 + (prand() % 3);
 
-			/*fgets(line, MAX_LINE_LENGTH, fp);*/
+						map.tile[y + 1][x] = map.tile[y][x] + 3;
+					}
+				}
+			}
+			*/
 		}
 
 		else if (loadEntityResources == TRUE && strcmpignorecase(itemName, "{") == 0)
@@ -266,14 +371,92 @@ void loadMap(char *name, int loadEntityResources)
 
 		else if (loadEntityResources == TRUE && strcmpignorecase(itemName, "}") == 0)
 		{
-			printf("Parse error: encountered closing } without matching {\n");
-
-			exit(1);
+			showErrorAndExit("Parse error: encountered closing } without matching {");
 		}
 
 		line = strtok_r(NULL, "\n", &savePtr1);
 	}
+	/*
+	printf("X %d - %d Y: %d - %d\n", map.minX, map.maxX, map.minY, map.maxY);
 
+	yy = 0;
+
+	if (map.minY != 0 || map.minX != 0)
+	{
+		for (y=map.minY;y<=map.maxY;y++)
+		{
+			xx = 0;
+
+			for (x=map.minX;x<=map.maxX;x++)
+			{
+				map.tile[yy][xx] = map.tile[y][x];
+
+				xx++;
+
+				map.tile[y][x] = 0;
+			}
+
+			yy++;
+		}
+
+
+		map.minX *= TILE_SIZE;
+		map.minY *= TILE_SIZE;
+
+		printf("MAP_NAME %s\nTRANSLATE_ENTITIES %d %d\n", name, map.minX, map.minY);
+
+		entities = getEntities();
+
+		player.x -= map.minX;
+		player.y -= map.minY;
+
+		for (el=entities->next;el!=NULL;el=el->next)
+		{
+			e = el->entity;
+
+			e->x -= map.minX;
+			e->y -= map.minY;
+
+			if (e->startX - map.minX > 0)
+			{
+				e->startX -= map.minX;
+			}
+
+			if (e->startY - map.minY > 0)
+			{
+				e->startY -= map.minY;
+			}
+
+			if (e->endX - map.minX > 0)
+			{
+				e->endX -= map.minX;
+			}
+
+			if (e->endY - map.minY > 0)
+			{
+				e->endY -= map.minY;
+			}
+		}
+
+		targets = getTargets();
+
+		for (x=0;x<MAX_TARGETS;x++)
+		{
+			if (targets[x].active == TRUE)
+			{
+				if (targets[x].x - map.minX > 0)
+				{
+					targets[x].x -= map.minX;
+				}
+
+				if (targets[x].y - map.minY > 0)
+				{
+					targets[x].y -= map.minY;
+				}
+			}
+		}
+	}
+	*/
 	/* Set the maximum scroll position of the map */
 
 	map.maxX = (map.maxX + 1) * TILE_SIZE;
@@ -298,14 +481,17 @@ void loadMap(char *name, int loadEntityResources)
 
 	setTransition(TRANSITION_IN, NULL);
 
-	playMusic();
+	if (game.canContinue == FALSE && game.overrideMusic == FALSE)
+	{
+		playMapMusic();
+	}
 
 	resetCameraLimits();
 
 	cameraSnapToTargetEntity();
 }
 
-void saveMap()
+int saveMap()
 {
 	int x, y;
 	char filename[MAX_LINE_LENGTH];
@@ -317,7 +503,7 @@ void saveMap()
 	{
 		printf("No player start defined\n");
 
-		return;
+		return FALSE;
 	}
 
 	snprintf(filename, sizeof(filename), "data/maps/%s.dat", map.filename);
@@ -330,9 +516,7 @@ void saveMap()
 
 	if (fp == NULL)
 	{
-		printf("Failed to open map %s\n", map.filename);
-
-		exit(1);
+		showErrorAndExit("Failed to open map %s", map.filename);
 	}
 
 	fprintf(fp, "MAP_NAME %s\n", map.mapName);
@@ -346,6 +530,23 @@ void saveMap()
 	fprintf(fp, "WRAP_X_2 %s\n", map.wrapX[1] == TRUE ? "TRUE" : "FALSE");
 	fprintf(fp, "WRAP_Y_2 %s\n", map.wrapY[1] == TRUE ? "TRUE" : "FALSE");
 	fprintf(fp, "WEATHER %s\n", getWeather());
+	fprintf(fp, "MIN_Y %s\n", map.forceMinY == TRUE ? "TRUE" : "FALSE");
+	fprintf(fp, "DARK_MAP %s\n", map.darkMap == TRUE ? "TRUE" : "FALSE");
+	fprintf(fp, "ANIM_TILE_COUNT %d\n", map.animTileTotal);
+
+	if (map.animTileTotal != 0)
+	{
+		for (x=0;x<map.animTileTotal;x++)
+		{
+			for (y=0;y<map.animTile[x].tileCount;y++)
+			{
+				fprintf(fp, "%d ", map.animTile[x].tile[y]);
+			}
+
+			fprintf(fp, "\n");
+		}
+	}
+
 	fprintf(fp, "DATA\n");
 
 	/* Write the data from the file into the map */
@@ -377,10 +578,16 @@ void saveMap()
 	/* Close the file afterwards */
 
 	fclose(fp);
+
+	return TRUE;
 }
 
 void doMap()
 {
+	int i;
+
+	i = 0;
+
 	map.thinkTime--;
 
 	if (map.thinkTime <= 0)
@@ -408,6 +615,30 @@ void doMap()
 
 		map.thinkTime = 3;
 	}
+
+	map.blendTime -= 3;
+
+	if (map.blendTime < 0)
+	{
+		map.blendTime = 0;
+	}
+
+	map.animThinkTime--;
+
+	if (map.animThinkTime < 0)
+	{
+		for (i=0;i<map.animTileTotal;i++)
+		{
+			map.animTile[i].tileIndex++;
+
+			if (map.animTile[i].tileIndex >= map.animTile[i].tileCount)
+			{
+				map.animTile[i].tileIndex = 0;
+			}
+		}
+
+		map.animThinkTime = 15;
+	}
 }
 
 static void loadMapTiles(char *dir)
@@ -429,11 +660,6 @@ static void loadMapTiles(char *dir)
 		}
 
 		mapImages[i] = loadImage(filename);
-
-		if (i >= WATER_TILE_START && i <= WATER_TILE_END)
-		{
-			SDL_SetAlpha(mapImages[i], SDL_SRCALPHA|SDL_RLEACCEL, 128);
-		}
 	}
 
 	snprintf(filename, sizeof(filename), "gfx/map/%s/background.png", dir);
@@ -445,122 +671,117 @@ static void loadMapTiles(char *dir)
 	loadMapBackground(filename, 1);
 }
 
-void drawMap(int depth)
+void drawMapBackground()
 {
-	int x, y, mapX, x1, x2, mapY, y1, y2, tileID;
+	int x, y;
 
-	/* Draw the background */
+	map.backgroundStartX[0] = map.startX * map.backgroundSpeed[0];
+	map.backgroundStartY[0] = map.startY * map.backgroundSpeed[0];
 
-	if (depth <= 0)
+	if (map.backgroundStartX[0] + SCREEN_WIDTH > map.background[0]->w && map.wrapX[0] == FALSE)
 	{
-		map.backgroundStartX[0] = map.startX * map.backgroundSpeed[0];
-		map.backgroundStartY[0] = map.startY * map.backgroundSpeed[0];
+		map.backgroundStartX[0] = map.background[0]->w - SCREEN_WIDTH;
+	}
 
-		if (map.backgroundStartX[0] + SCREEN_WIDTH > map.background[0]->w && map.wrapX[0] == FALSE)
+	if (map.backgroundStartY[0] + SCREEN_HEIGHT > map.background[0]->h && map.wrapY[0] == FALSE)
+	{
+		map.backgroundStartY[0] = map.background[0]->h - SCREEN_HEIGHT;
+	}
+
+	if (map.wrapX[0] == FALSE && map.wrapY[0] == FALSE)
+	{
+		drawClippedImage(map.background[0], map.backgroundStartX[0], map.backgroundStartY[0], 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
+	}
+
+	else if (map.wrapX[0] == TRUE && map.wrapY[0] == FALSE)
+	{
+		x = map.backgroundStartX[0] % map.background[0]->w;
+
+		drawClippedImage(map.background[0], x, map.backgroundStartY[0], 0, 0, map.background[0]->w - x, SCREEN_HEIGHT);
+
+		drawClippedImage(map.background[0], 0, map.backgroundStartY[0], map.background[0]->w - x, 0, x, SCREEN_HEIGHT);
+	}
+
+	else if (map.wrapX[0] == FALSE && map.wrapY[0] == TRUE)
+	{
+		y = map.backgroundStartY[0] % map.background[0]->h;
+
+		drawClippedImage(map.background[0], map.backgroundStartX[0], y, 0, 0, SCREEN_WIDTH, map.background[0]->h - y);
+
+		drawClippedImage(map.background[0], map.backgroundStartX[0], 0, 0, map.background[0]->h - y, SCREEN_WIDTH, y);
+	}
+
+	else
+	{
+		x = map.backgroundStartX[0] % map.background[0]->w;
+		y = map.backgroundStartY[0] % map.background[0]->h;
+
+		drawClippedImage(map.background[0], x, y, 0, 0, map.background[0]->w - x, map.background[0]->h - y);
+
+		drawClippedImage(map.background[0], 0, y, map.background[0]->w - x, 0, x, map.background[0]->h - y);
+
+		drawClippedImage(map.background[0], x, 0, 0, map.background[0]->h - y, map.background[0]->w - x, y);
+
+		drawClippedImage(map.background[0], 0, 0, map.background[0]->w - x, map.background[0]->h - y, x, y);
+	}
+
+	if (map.background[1] != NULL)
+	{
+		map.backgroundStartX[1] = map.startX * map.backgroundSpeed[1];
+		map.backgroundStartY[1] = map.startY * map.backgroundSpeed[1];
+
+		if (map.backgroundStartX[1] + SCREEN_WIDTH > map.background[1]->w && map.wrapX[1] == FALSE)
 		{
-			map.backgroundStartX[0] = map.background[0]->w - SCREEN_WIDTH;
+			map.backgroundStartX[1] = map.background[1]->w - SCREEN_WIDTH;
 		}
 
-		if (map.backgroundStartY[0] + SCREEN_HEIGHT > map.background[0]->h && map.wrapY[0] == FALSE)
+		if (map.backgroundStartY[1] + SCREEN_HEIGHT > map.background[1]->h && map.wrapY[1] == FALSE)
 		{
-			map.backgroundStartY[0] = map.background[0]->h - SCREEN_HEIGHT;
+			map.backgroundStartY[1] = map.background[1]->h - SCREEN_HEIGHT;
 		}
 
-		if (map.wrapX == FALSE && map.wrapY == FALSE)
+		if (map.wrapX[1] == FALSE && map.wrapY[1] == FALSE)
 		{
-			drawClippedImage(map.background[0], map.backgroundStartX[0], map.backgroundStartY[0], 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
+			drawClippedImage(map.background[1], map.backgroundStartX[1], map.backgroundStartY[1], 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
 		}
 
-		else if (map.wrapX[0] == TRUE && map.wrapY[0] == FALSE)
+		else if (map.wrapX[1] == TRUE && map.wrapY[1] == FALSE)
 		{
-			x = map.backgroundStartX[0] % map.background[0]->w;
+			x = map.backgroundStartX[1] % map.background[1]->w;
 
-			drawClippedImage(map.background[0], x, map.backgroundStartY[0], 0, 0, map.background[0]->w - x, SCREEN_HEIGHT);
+			drawClippedImage(map.background[1], x, map.backgroundStartY[1], 0, 0, map.background[1]->w - x, SCREEN_HEIGHT);
 
-			drawClippedImage(map.background[0], 0, map.backgroundStartY[0], map.background[0]->w - x, 0, x, SCREEN_HEIGHT);
+			drawClippedImage(map.background[1], 0, map.backgroundStartY[1], map.background[1]->w - x, 0, x, SCREEN_HEIGHT);
 		}
 
-		else if (map.wrapX[0] == FALSE && map.wrapY[0] == TRUE)
+		else if (map.wrapX[1] == FALSE && map.wrapY[1] == TRUE)
 		{
-			y = map.backgroundStartY[0] % map.background[0]->h;
+			y = map.backgroundStartY[1] % map.background[1]->h;
 
-			drawClippedImage(map.background[0], map.backgroundStartX[0], y, 0, 0, SCREEN_WIDTH, map.background[0]->h - y);
+			drawClippedImage(map.background[1], map.backgroundStartX[1], y, 0, 0, SCREEN_WIDTH, map.background[1]->h - y);
 
-			drawClippedImage(map.background[0], map.backgroundStartX[0], 0, 0, map.background[0]->h - y, SCREEN_WIDTH, y);
+			drawClippedImage(map.background[1], map.backgroundStartX[1], 0, 0, map.background[1]->h - y, SCREEN_WIDTH, y);
 		}
 
 		else
 		{
-			x = map.backgroundStartX[0] % map.background[0]->w;
-			y = map.backgroundStartY[0] % map.background[0]->h;
+			x = map.backgroundStartX[1] % map.background[1]->w;
+			y = map.backgroundStartY[1] % map.background[1]->h;
 
-			drawClippedImage(map.background[0], x, y, 0, 0, map.background[0]->w - x, map.background[0]->h - y);
+			drawClippedImage(map.background[1], x, y, 0, 0, map.background[1]->w - x, map.background[1]->h - y);
 
-			drawClippedImage(map.background[0], 0, y, map.background[0]->w - x, 0, x, map.background[0]->h - y);
+			drawClippedImage(map.background[1], 0, y, map.background[1]->w - x, 0, x, map.background[1]->h - y);
 
-			drawClippedImage(map.background[0], x, 0, 0, map.background[0]->h - y, map.background[0]->w - x, y);
+			drawClippedImage(map.background[1], x, 0, 0, map.background[1]->h - y, map.background[1]->w - x, y);
 
-			drawClippedImage(map.background[0], 0, 0, map.background[0]->w - x, map.background[0]->h - y, x, y);
-		}
-
-		if (map.background[1] != NULL)
-		{
-			map.backgroundStartX[1] = map.startX * map.backgroundSpeed[1];
-			map.backgroundStartY[1] = map.startY * map.backgroundSpeed[1];
-
-			if (map.backgroundStartX[1] + SCREEN_WIDTH > map.background[1]->w && map.wrapX[1] == FALSE)
-			{
-				map.backgroundStartX[1] = map.background[1]->w - SCREEN_WIDTH;
-			}
-
-			if (map.backgroundStartY[1] + SCREEN_HEIGHT > map.background[1]->h && map.wrapY[1] == FALSE)
-			{
-				map.backgroundStartY[1] = map.background[1]->h - SCREEN_HEIGHT;
-			}
-
-			if (map.wrapX[1] == FALSE && map.wrapY[1] == FALSE)
-			{
-				drawClippedImage(map.background[1], map.backgroundStartX[1], map.backgroundStartY[1], 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
-			}
-
-			else if (map.wrapX[1] == TRUE && map.wrapY[1] == FALSE)
-			{
-				x = map.backgroundStartX[1] % map.background[1]->w;
-
-				drawClippedImage(map.background[1], x, map.backgroundStartY[1], 0, 0, map.background[1]->w - x, SCREEN_HEIGHT);
-
-				drawClippedImage(map.background[1], 0, map.backgroundStartY[1], map.background[1]->w - x, 0, x, SCREEN_HEIGHT);
-			}
-
-			else if (map.wrapX[1] == FALSE && map.wrapY[1] == TRUE)
-			{
-				y = map.backgroundStartY[1] % map.background[1]->h;
-
-				drawClippedImage(map.background[1], map.backgroundStartX[1], y, 0, 0, SCREEN_WIDTH, map.background[1]->h - y);
-
-				drawClippedImage(map.background[1], map.backgroundStartX[1], 0, 0, map.background[1]->h - y, SCREEN_WIDTH, y);
-			}
-
-			else
-			{
-				x = map.backgroundStartX[1] % map.background[1]->w;
-				y = map.backgroundStartY[1] % map.background[1]->h;
-
-				drawClippedImage(map.background[1], x, y, 0, 0, map.background[1]->w - x, map.background[1]->h - y);
-
-				drawClippedImage(map.background[1], 0, y, map.background[1]->w - x, 0, x, map.background[1]->h - y);
-
-				drawClippedImage(map.background[1], x, 0, 0, map.background[1]->h - y, map.background[1]->w - x, y);
-
-				drawClippedImage(map.background[1], 0, 0, map.background[1]->w - x, map.background[1]->h - y, x, y);
-			}
+			drawClippedImage(map.background[1], 0, 0, map.background[1]->w - x, map.background[1]->h - y, x, y);
 		}
 	}
+}
 
-	if (depth == 0)
-	{
-		return;
-	}
+void drawMap(int depth)
+{
+	int i, x, y, mapX, x1, x2, mapY, y1, y2, tileID;
 
 	mapX = map.startX / TILE_SIZE;
 	x1 = (map.startX % TILE_SIZE) * -1;
@@ -597,12 +818,23 @@ void drawMap(int depth)
 				break;
 			}
 
+			i = 0;
+
+			for (i=0;i<map.animTileTotal;i++)
+			{
+				if (tileID == map.animTile[i].tile[0])
+				{
+					tileID = map.animTile[i].tile[map.animTile[i].tileIndex];
+				}
+			}
+
 			if (tileID == BLANK_TILE)
 			{
 				mapX++;
 
 				continue;
 			}
+
 			/*
 			if (mapImages[tileID] == NULL)
 			{
@@ -611,24 +843,62 @@ void drawMap(int depth)
 				exit(0);
 			}
 			*/
-			if (depth == -1)
+
+			if (game.status == IN_CREDITS)
 			{
-				drawImage(mapImages[tileID], x, y, FALSE);
+				drawBox(game.screen, x, y, TILE_SIZE, TILE_SIZE, 0, 0, 0);
 			}
 
-			else if (depth == 1)
+			else
 			{
-				if (tileID < FOREGROUND_TILE_START)
+				switch (depth)
 				{
-					drawImage(mapImages[tileID], x, y, FALSE);
-				}
-			}
+					case 0:
+						if (tileID >= BACKGROUND_TILE_START && tileID <= BACKGROUND_TILE_END)
+						{
+							drawImage(mapImages[tileID], x, y, FALSE, 255);
+						}
+					break;
 
-			else if (depth == 2)
-			{
-				if (tileID >= FOREGROUND_TILE_START)
-				{
-					drawImage(mapImages[tileID], x, y, FALSE);
+					case 1:
+						if (tileID < BACKGROUND_TILE_START)
+						{
+							drawImage(mapImages[tileID], x, y, FALSE, 255);
+						}
+					break;
+
+					case 2:
+						if (tileID == SLIME_TILE_BLEND)
+						{
+							drawImage(mapImages[WATER_TILE_START], x, y, FALSE, 128);
+							drawImage(mapImages[slimeTile], x, y, FALSE, map.blendTime);
+
+							if (map.blendTime == 0)
+							{
+								map.tile[mapY][mapX] = WATER_TILE_START;
+							}
+						}
+
+						else if (tileID == SLIME_TILE_BLEND_REVERSE)
+						{
+							drawImage(mapImages[WATER_TILE_START], x, y, FALSE, 128);
+							drawImage(mapImages[slimeTile], x, y, FALSE, 255 - map.blendTime);
+
+							if (map.blendTime == 0)
+							{
+								map.tile[mapY][mapX] = SLIME_TILE_START;
+							}
+						}
+
+						else if (tileID >= FOREGROUND_TILE_START)
+						{
+							drawImage(mapImages[tileID], x, y, FALSE, tileID >= WATER_TILE_START && tileID <= WATER_TILE_END ? 128 : 255);
+						}
+					break;
+
+					default:
+						drawImage(mapImages[tileID], x, y, FALSE, 255);
+					break;
 				}
 			}
 
@@ -652,9 +922,9 @@ void centerEntityOnMap()
 	{
 		speed = map.targetEntity->standingOn->speed;
 
-		if (speed < fabs(map.targetEntity->standingOn->dirX))
+		if (speed < fabs(map.targetEntity->dirX))
 		{
-			speed = fabs(map.targetEntity->standingOn->dirX);
+			speed = fabs(map.targetEntity->dirX);
 		}
 
 		if (speed < fabs(map.targetEntity->standingOn->dirY))
@@ -866,42 +1136,42 @@ int mapTileAt(int x, int y)
 	return map.tile[y][x];
 }
 
-int maxMapX()
+int getMapMaxX()
 {
 	return map.maxX;
 }
 
-int maxMapY()
+int getMapMaxY()
 {
 	return map.maxY;
 }
 
-void setMaxMapX(int max)
+void setMapMaxX(int max)
 {
 	map.maxX = max;
 }
 
-void setMaxMapY(int max)
+void setMapMaxY(int max)
 {
 	map.maxY = max;
 }
 
-void setMinMapX(int min)
+void setMapMinX(int min)
 {
 	map.minX = min;
 }
 
-void setMinMapY(int min)
+void setMapMinY(int min)
 {
 	map.minY = min;
 }
 
-int getMinMapX()
+int getMapMinX()
 {
 	return map.minX;
 }
 
-int getMinMapY()
+int getMapMinY()
 {
 	return map.minY;
 }
@@ -1064,6 +1334,19 @@ void setCameraPosition(int x, int y)
 {
 	map.cameraX = x;
 	map.cameraY = y;
+
+	map.startX = map.cameraX;
+	map.startY = map.cameraY;
+}
+
+void setCameraPositionFromScript(char *position)
+{
+	char *x, *y;
+
+	x = strtok(position, " ");
+	y = strtok(NULL, " ");
+
+	setCameraPosition(atoi(x), atoi(y));
 }
 
 void limitCamera(int minX, int minY, int maxX, int maxY)
@@ -1077,7 +1360,28 @@ void limitCamera(int minX, int minY, int maxX, int maxY)
 
 void limitCameraFromScript(char *line)
 {
-	sscanf(line, "%d %d %d %d", &map.cameraMinX, &map.cameraMinY, &map.cameraMaxX, &map.cameraMaxY);
+	char limitPlayer[10];
+	int read;
+
+	read = sscanf(line, "%d %d %d %d %s", &map.cameraMinX, &map.cameraMinY, &map.cameraMaxX, &map.cameraMaxY, limitPlayer);
+
+	if (read == 5 && strcmpignorecase(limitPlayer, "TRUE") == 0)
+	{
+		map.playerMinX = map.cameraMinX;
+		map.playerMaxX = map.cameraMaxX;
+	}
+
+	else
+	{
+		map.playerMinX = map.minX;
+		map.playerMaxX = map.maxX;
+	}
+}
+
+void limitPlayerToCameraLimits()
+{
+	map.playerMinX = map.cameraMinX;
+	map.playerMaxX = map.cameraMaxX;
 }
 
 void resetCameraLimits()
@@ -1087,6 +1391,29 @@ void resetCameraLimits()
 
 	map.cameraMaxX = map.maxX;
 	map.cameraMaxY = map.maxY;
+
+	map.playerMinX = map.minX;
+	map.playerMaxX = map.maxX;
+}
+
+int getCameraMinX()
+{
+	return map.cameraMinX;
+}
+
+int getCameraMinY()
+{
+	return map.cameraMinY;
+}
+
+int getCameraMaxX()
+{
+	return map.cameraMaxX;
+}
+
+int getCameraMaxY()
+{
+	return map.cameraMaxY;
 }
 
 int cameraAtMinimum()
@@ -1094,6 +1421,16 @@ int cameraAtMinimum()
 	return (map.cameraMinX == map.startX && map.cameraMinY == map.startY);
 }
 
+int getPlayerMinX()
+{
+	return map.playerMinX;
+}
+
+int getPlayerMaxX()
+{
+	return map.playerMaxX;
+}
+
 void setCameraSpeed(float speed)
 {
 	map.cameraSpeed = speed;
@@ -1125,3 +1462,18 @@ int getDistanceFromCamera(int x, int y)
 {
 	return getDistance(map.cameraX + SCREEN_WIDTH / 2, map.cameraY + SCREEN_HEIGHT / 2, x, y);
 }
+
+void setDarkMap(int dark)
+{
+	map.darkMap = dark;
+}
+
+int isDarkMap()
+{
+	return map.darkMap;
+}
+
+void resetBlendTime()
+{
+	map.blendTime = 255;
+}
