@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 Entity *loadPlayer(int, int, char *);
@@ -35,8 +35,29 @@ void writePlayerMapStartToFile(FILE *);
 void increasePlayerMaxHealth(void);
 void syncWeaponShieldToPlayer(void);
 void freePlayer(void);
-void resetPlayerAnimations(void);
-void playerGib(void);
+EntityList *playerGib(void);
 void facePlayer(void);
 void setPlayerSlimed(int);
 void setBowAmmo(int);
+void setPlayerStunned(int);
+void doStunned(void);
+void becomeJumpingSlime(int);
+void becomeEdgar(void);
+void setPlayerLocked(int);
+Entity *removePlayerWeapon(void);
+Entity *removePlayerShield(void);
+void setPlayerFrozen(int);
+void setPlayerWrapped(int);
+int isAttacking(void);
+void playerStand(void);
+void addChargesToWeapon(void);
+int isPlayerLocked(void);
+void playerWaitForConfirm(void);
+void setWeaponFromScript(char *);
+void scriptAttack(void);
+void setShieldFromScript(char *);
+void unsetWeapon(void);
+void unsetShield(void);
+void setPlayerConfused(int);
+void setPlayerPetrified(void);
+void setPlayerAsh(void);
