@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,57 +14,52 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 #include "../headers.h"
 
+#include "../audio/music.h"
 #include "../entity.h"
-#include "../event/trigger.h"
 #include "../event/global_trigger.h"
+#include "../event/map_trigger.h"
 #include "../event/objective.h"
-#include "../world/target.h"
-#include "../map.h"
-#include "../player.h"
-#include "../inventory.h"
-#include "compress.h"
-#include "resources.h"
+#include "../event/script.h"
+#include "../event/trigger.h"
 #include "../game.h"
-#include "load_save.h"
+#include "../graphics/font.h"
+#include "../graphics/graphics.h"
 #include "../hud.h"
-#include "pak.h"
 #include "../input.h"
+#include "../inventory.h"
+#include "../map.h"
+#include "../medal.h"
+#include "../player.h"
+#include "../world/target.h"
+#include "compress.h"
+#include "error.h"
+#include "load_save.h"
+#include "resources.h"
 
-static char gameSavePath[MAX_PATH_LENGTH], tempFile[MAX_PATH_LENGTH], saveFileIndex[MAX_PATH_LENGTH];
+static char gameSavePath[MAX_PATH_LENGTH], tempFile[MAX_PATH_LENGTH], saveFileIndex[MAX_PATH_LENGTH], continueFile[MAX_PATH_LENGTH];
+static int temporaryDataExists;
 
 static void removeTemporaryData(void);
 static void copyFile(char *, char *);
 static void updateSaveFileIndex(int);
 static void patchSaveGame(char *, double);
+static void showPatchMessage(char *);
 
 extern Game game;
 
-#ifndef WIN32
+#if UNIX == 1
 	void setupUserHomeDirectory()
 	{
 		char *userHome;
-		char *name;
 		char dir[MAX_PATH_LENGTH];
 		struct passwd *pass;
 
-		name = getlogin();
-
-		STRNCPY(dir, "", sizeof(dir));
-
-		if (name != NULL)
-		{
-			pass = getpwnam(name);
-		}
-
-		else
-		{
-			pass = getpwuid(geteuid());
-		}
+		pass = getpwuid(geteuid());
 
 		if (pass == NULL)
 		{
@@ -75,7 +70,11 @@ extern Game game;
 
 		userHome = pass->pw_dir;
 
+		#if MACOS == 1
+		snprintf(dir, sizeof(dir), "%s/Library/Application Support", userHome);
+		#else
 		snprintf(dir, sizeof(dir), "%s/.parallelrealities", userHome);
+		#endif
 
 		if ((mkdir(dir, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH) != 0) && (errno != EEXIST))
 		{
@@ -84,7 +83,11 @@ extern Game game;
 			exit(1);
 		}
 
+		#if MACOS == 1
+		snprintf(dir, sizeof(dir), "%s/Library/Application Support/Edgar", userHome);
+		#else
 		snprintf(dir, sizeof(dir), "%s/.parallelrealities/edgar", userHome);
+		#endif
 
 		if ((mkdir(dir, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH) != 0) && (errno != EEXIST))
 		{
@@ -93,13 +96,17 @@ extern Game game;
 			exit(1);
 		}
 
+		#if MACOS == 1
+		snprintf(gameSavePath, sizeof(gameSavePath), "%s/Library/Application Support/Edgar/", userHome);
+		#else
 		snprintf(gameSavePath, sizeof(gameSavePath), "%s/.parallelrealities/edgar/", userHome);
+		#endif
 
 		snprintf(tempFile, sizeof(tempFile), "%stmpsave", gameSavePath);
 
 		snprintf(saveFileIndex, sizeof(saveFileIndex), "%ssaveheader", gameSavePath);
 
-		removeTemporaryData();
+		snprintf(continueFile, sizeof(continueFile), "%scontinuesave", gameSavePath);
 	}
 #else
 	void setupUserHomeDirectory()
@@ -107,33 +114,76 @@ extern Game game;
 		STRNCPY(gameSavePath, "", sizeof(gameSavePath));
 		STRNCPY(tempFile, "tmpsave", sizeof(tempFile));
 		STRNCPY(saveFileIndex, "saveheader", sizeof(saveFileIndex));
-
-		removeTemporaryData();
+		STRNCPY(continueFile, "continuesave", sizeof(continueFile));
 	}
 #endif
 
+void startOnMap(char *mapName)
+{
+	removeTemporaryData();
+
+	freeGameResources();
+
+	initGame();
+
+	loadMap(mapName, TRUE);
+
+	cameraSnapToTargetEntity();
+}
+
 void newGame()
 {
 	removeTemporaryData();
 
 	freeGameResources();
 
-	loadMap("map01", TRUE);
+	initGame();
+
+	stopMusic();
+
+	game.overrideMusic = TRUE;
+
+	loadMap("map04", TRUE);
+
+	runScript("intro_part1");
+}
+
+void tutorial()
+{
+	removeTemporaryData();
+
+	freeGameResources();
 
 	initGame();
 
+	loadMap("tutorial", TRUE);
+
 	cameraSnapToTargetEntity();
 }
 
+void titleScreen()
+{
+	removeTemporaryData();
+
+	freeGameResources();
+
+	initGame();
+
+	game.status = IN_TITLE;
+}
+
 int loadGame(int slot)
 {
 	char itemName[MAX_MESSAGE_LENGTH], mapName[MAX_MESSAGE_LENGTH], backup[MAX_PATH_LENGTH];
-	char saveFile[MAX_PATH_LENGTH], *line, *savePtr;
-	double version;
+	char saveFile[MAX_PATH_LENGTH], *line, *savePtr, completion[5];
+	double version = 0;
+	float percentage, steps;
 	unsigned char *buffer;
-	int patchGame = FALSE;
+	int patchGame = FALSE, foundResources;
 	FILE *fp;
 
+	savePtr = NULL;
+
 	snprintf(saveFile, sizeof(saveFile), "%ssave%d", gameSavePath, slot);
 
 	fp = fopen(saveFile, "rb");
@@ -147,14 +197,14 @@ int loadGame(int slot)
 
 	freeGameResources();
 
-	printf("Loading save data from %s\n", saveFile);
+	initGame();
 
 	buffer = decompressFile(saveFile);
 
-	printf("Reading save data\n");
-
 	line = strtok_r((char *)buffer, "\n", &savePtr);
 
+	foundResources = FALSE;
+
 	while (line != NULL)
 	{
 		if (line[strlen(line) - 1] == '\n')
@@ -175,15 +225,9 @@ int loadGame(int slot)
 
 			version = atof(itemName);
 
-			printf("VERSION IS %0.2f\n", version);
-
-			if (version < VERSION)
+			if (version > VERSION)
 			{
-				printf("Need to patch save file\n");
-
-				patchGame = TRUE;
-
-				break;
+				printf("Save file version is newer than game version. This game might not work correctly.\n");
 			}
 		}
 
@@ -192,11 +236,66 @@ int loadGame(int slot)
 			sscanf(line, "%*s %ld\n", &game.playTime);
 		}
 
+		else if (strcmpignorecase("PLAYER_KILLS", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.kills);
+		}
+
+		else if (strcmpignorecase("BATS_DROWNED", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.batsDrowned);
+		}
+
+		else if (strcmpignorecase("TIMES_EATEN", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.timesEaten);
+		}
+
+		else if (strcmpignorecase("DISTANCE_TRAVELLED", itemName) == 0)
+		{
+			sscanf(line, "%*s %u\n", &game.distanceTravelled);
+		}
+
+		else if (strcmpignorecase("ATTACKS_BLOCKED", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.attacksBlocked);
+		}
+
+		else if (strcmpignorecase("SLIME_TIME", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.timeSpentAsSlime);
+		}
+
+		else if (strcmpignorecase("ARROWS_FIRED", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.arrowsFired);
+		}
+
+		else if (strcmpignorecase("SECRETS_FOUND", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.secretsFound);
+		}
+
+		else if (strcmpignorecase("CONTINUES", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.continues);
+		}
+
+		else if (strcmpignorecase("CHEATING", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.cheating);
+		}
+
 		else if (strcmpignorecase("PLAYER_LOCATION", itemName) == 0)
 		{
-			sscanf(line, "%*s %s\n", itemName);
+			if (version < VERSION)
+			{
+				patchGame = TRUE;
 
-			printf("Loading save location %s\n", itemName);
+				break;
+			}
+
+			sscanf(line, "%*s %s\n", itemName);
 
 			loadMap(itemName, FALSE);
 
@@ -205,7 +304,7 @@ int loadGame(int slot)
 
 		else if (strcmpignorecase(line, mapName) == 0)
 		{
-			printf("Loading entities for map %s\n", mapName);
+			foundResources = TRUE;
 
 			loadResources(savePtr);
 		}
@@ -217,6 +316,12 @@ int loadGame(int slot)
 	{
 		free(buffer);
 
+		steps = (VERSION * 100) - (version * 100);
+
+		steps = 100 / steps;
+
+		percentage = 0;
+
 		version += 0.01;
 
 		/* Back up the original save file */
@@ -225,35 +330,40 @@ int loadGame(int slot)
 
 		copyFile(saveFile, backup);
 
-		printf("Backed up original save to %s\n", backup);
+		showPatchMessage("0%");
 
 		while (TRUE)
 		{
+			getInput(IN_TITLE);
+
 			patchSaveGame(saveFile, version);
 
 			version += 0.01;
-			
-			if (version > VERSION)
+
+			percentage += steps;
+
+			snprintf(completion, 5, "%d%%", (int)percentage);
+
+			if ((int)(version * 100) > (int)(VERSION * 100))
 			{
 				break;
 			}
+
+			showPatchMessage(completion);
 		}
 
-		printf("Patch completed. Replacing save game.\n");
-
-		#if DEV == 1
-			copyFile(tempFile, "tmpdata");
-		#endif
-
-		copyFile(tempFile, saveFile);
-
-		compressFile(saveFile);
-
-		printf("Loading new save\n");
-
 		return loadGame(slot);
 	}
 
+	/* Fudge to make a game saved in the new Village map still load OK */
+
+	if (foundResources == FALSE)
+	{
+		sscanf(mapName, "%*s %s\n", itemName);
+
+		loadMap(itemName, TRUE);
+	}
+
 	free(buffer);
 
 	copyFile(saveFile, tempFile);
@@ -266,7 +376,7 @@ int loadGame(int slot)
 
 	freeMessageQueue();
 
-	printf("Load completed\n");
+	temporaryDataExists = TRUE;
 
 	return TRUE;
 }
@@ -275,30 +385,26 @@ static void patchSaveGame(char *saveFile, double version)
 {
 	char location[MAX_VALUE_LENGTH], *line, *savePtr, itemName[MAX_MESSAGE_LENGTH], *returnedName, mapName[MAX_VALUE_LENGTH];
 	unsigned char *buffer, *originalBuffer;
-	int savedLocation = FALSE;
+	int savedLocation = FALSE, saveMap;
 	FILE *newSave;
 
+	savePtr = NULL;
+
 	freeGameResources();
 
 	newSave = fopen(tempFile, "wb");
 
-	printf("PATCHING save data from %s\n", saveFile);
-
 	buffer = decompressFile(saveFile);
 
-	originalBuffer = (unsigned char *)malloc((strlen((char *)buffer) + 1) * sizeof(unsigned char));
+	originalBuffer = malloc((strlen((char *)buffer) + 1) * sizeof(unsigned char));
 
 	if (originalBuffer == NULL)
 	{
-		printf("Failed to allocate %d bytes to patch save game\n", (int)((strlen((char *)buffer) + 1) * sizeof(unsigned char)));
-
-		exit(0);
+		showErrorAndExit("Failed to allocate %d bytes to patch save game", (int)((strlen((char *)buffer) + 1) * sizeof(unsigned char)));
 	}
 
 	strcpy((char *)originalBuffer, (char *)buffer);
 
-	printf("Reading save data\n");
-
 	line = strtok_r((char *)buffer, "\n", &savePtr);
 
 	returnedName = NULL;
@@ -326,14 +432,12 @@ static void patchSaveGame(char *saveFile, double version)
 
 		else if (strcmpignorecase("MAP_NAME", itemName) == 0)
 		{
+			saveMap = TRUE;
+
 			if (strlen(mapName) == 0 || strcmpignorecase(line, mapName) == 0)
 			{
 				sscanf(line, "%*s %s\n", itemName);
 
-				printf("Will patch %s\n", itemName);
-
-				printf("Loading resources for %s\n", itemName);
-
 				returnedName = loadResources(savePtr);
 
 				if (returnedName != NULL)
@@ -341,35 +445,56 @@ static void patchSaveGame(char *saveFile, double version)
 					STRNCPY(mapName, returnedName, sizeof(mapName));
 				}
 
-				printf("Resources loaded for %s\n", itemName);
-
 				/* Rewind to start */
 
 				free(buffer);
 
-				buffer = (unsigned char *)malloc((strlen((char *)originalBuffer) + 1) * sizeof(unsigned char));
+				buffer = malloc((strlen((char *)originalBuffer) + 1) * sizeof(unsigned char));
 
 				if (buffer == NULL)
 				{
-					printf("Failed to allocate %d bytes to patch save game\n", (int)((strlen((char *)originalBuffer) + 1) * sizeof(unsigned char)));
-
-					exit(0);
+					showErrorAndExit("Failed to allocate %d bytes to patch save game", (int)((strlen((char *)originalBuffer) + 1) * sizeof(unsigned char)));
 				}
 
 				strcpy((char *)buffer, (char *)originalBuffer);
 
 				line = strtok_r((char *)buffer, "\n", &savePtr);
 
-				printf("Applying patch to %s\n", itemName);
-
 				/* Load up the patch file */
 
-				patchEntities(version, itemName);
+				saveMap = patchEntities(version, itemName);
+
+				if (saveMap == FALSE)
+				{
+					snprintf(itemName, sizeof(itemName), "%ld_old", game.playTime * SDL_GetTicks());
+				}
 
 				if (savedLocation == FALSE)
 				{
 					fprintf(newSave, "VERSION %0.2f\n", version);
 
+					fprintf(newSave, "PLAY_TIME %ld\n", game.playTime);
+
+					fprintf(newSave, "PLAYER_KILLS %d\n", game.kills);
+
+					fprintf(newSave, "BATS_DROWNED %d\n", game.batsDrowned);
+
+					fprintf(newSave, "TIMES_EATEN %d\n", game.timesEaten);
+
+					fprintf(newSave, "DISTANCE_TRAVELLED %u\n", game.distanceTravelled);
+
+					fprintf(newSave, "ATTACKS_BLOCKED %d\n", game.attacksBlocked);
+
+					fprintf(newSave, "SLIME_TIME %d\n", game.timeSpentAsSlime);
+
+					fprintf(newSave, "ARROWS_FIRED %d\n", game.arrowsFired);
+
+					fprintf(newSave, "SECRETS_FOUND %d\n", game.secretsFound);
+
+					fprintf(newSave, "CONTINUES %d\n", game.continues);
+
+					fprintf(newSave, "CHEATING %d\n", game.cheating);
+
 					fprintf(newSave, "PLAYER_LOCATION %s\n", location);
 
 					savedLocation = TRUE;
@@ -406,6 +531,10 @@ static void patchSaveGame(char *saveFile, double version)
 
 				writeGlobalTriggersToFile(newSave);
 
+				/* Add the map triggers */
+
+				writeMapTriggersToFile(newSave);
+
 				/* Add the objectives */
 
 				writeObjectivesToFile(newSave);
@@ -427,36 +556,99 @@ static void patchSaveGame(char *saveFile, double version)
 	free(originalBuffer);
 
 	fclose(newSave);
+
+	#if DEV == 1
+		copyFile(tempFile, "tmpdata");
+	#endif
+
+	copyFile(tempFile, saveFile);
+
+	compressFile(saveFile);
 }
 
 void saveGame(int slot)
 {
-	char itemName[MAX_MESSAGE_LENGTH], *line, *savePtr, *playTimeString;
+	char itemName[MAX_MESSAGE_LENGTH], *line, *savePtr;
 	char saveFile[MAX_PATH_LENGTH];
 	char *mapName = getMapFilename();
+	char tempSaveFile[MAX_PATH_LENGTH];
+	int i;
 	unsigned char *buffer;
 	int skipping = FALSE;
 	FILE *read;
 	FILE *write;
 
-	printf("Saving game\n");
+	savePtr = NULL;
 
-	snprintf(saveFile, sizeof(saveFile), "%ssave%d", gameSavePath, slot);
+	if (slot == -1)
+	{
+		for (i=1001;;i++)
+		{
+			snprintf(saveFile, sizeof(saveFile), "%ssave%d", gameSavePath, i);
 
-	read = fopen(tempFile, "rb");
+			read = fopen(saveFile, "rb");
 
-	write = fopen(saveFile, "wb");
+			if (read == NULL)
+			{
+				break;
+			}
+
+			else
+			{
+				fclose(read);
+			}
+		}
+	}
+
+	else
+	{
+		/* Backup older save */
+
+		snprintf(saveFile, sizeof(saveFile), "%ssave%d", gameSavePath, slot);
+
+		snprintf(tempSaveFile, sizeof(tempSaveFile), "%stempsave%d", gameSavePath, slot);
+	}
+
+	write = fopen(tempSaveFile, "wb");
 
 	fprintf(write, "VERSION %0.2f\n", VERSION);
 
 	fprintf(write, "PLAY_TIME %ld\n", game.playTime);
 
+	fprintf(write, "PLAYER_KILLS %d\n", game.kills);
+
+	fprintf(write, "BATS_DROWNED %d\n", game.batsDrowned);
+
+	fprintf(write, "TIMES_EATEN %d\n", game.timesEaten);
+
+	fprintf(write, "DISTANCE_TRAVELLED %u\n", game.distanceTravelled);
+
+	fprintf(write, "ATTACKS_BLOCKED %d\n", game.attacksBlocked);
+
+	fprintf(write, "SLIME_TIME %d\n", game.timeSpentAsSlime);
+
+	fprintf(write, "ARROWS_FIRED %d\n", game.arrowsFired);
+
+	fprintf(write, "SECRETS_FOUND %d\n", game.secretsFound);
+
+	fprintf(write, "CONTINUES %d\n", game.continues);
+
+	fprintf(write, "CHEATING %d\n", game.cheating);
+
 	fprintf(write, "PLAYER_LOCATION %s\n", mapName);
 
-	if (read != NULL)
-	{
-		printf("Copying persisting data\n");
+	read = fopen(tempFile, "rb");
 
+	if (read == NULL)
+	{
+		if (temporaryDataExists == TRUE)
+		{
+			showErrorAndExit("SAVE GAME: Could not find persistance file: %s", strerror(errno));
+		}
+	}
+
+	else
+	{
 		fclose(read);
 
 		buffer = decompressFile(tempFile);
@@ -531,17 +723,13 @@ void saveGame(int slot)
 
 	/* Save the player's position */
 
-	printf("Writing player location\n");
-
 	fprintf(write, "MAP_NAME %s\n", mapName);
 
-	printf("Writing player data\n");
-
 	fprintf(write, "PLAYER_DATA\n");
 
 	writePlayerToFile(write);
 
-	printf("Writing player inventory\n");
+	fprintf(write, "INVENTORY_INDEX %d\n", getInventoryIndex());
 
 	fprintf(write, "PLAYER_INVENTORY\n");
 
@@ -565,6 +753,10 @@ void saveGame(int slot)
 
 	writeGlobalTriggersToFile(write);
 
+	/* Add the map triggers */
+
+	writeMapTriggersToFile(write);
+
 	/* Add the objectives */
 
 	writeObjectivesToFile(write);
@@ -574,18 +766,16 @@ void saveGame(int slot)
 	fclose(write);
 
 	#if DEV == 1
-		copyFile(saveFile, "savedata");
+		copyFile(tempSaveFile, "savedata");
 	#endif
 
-	compressFile(saveFile);
+	compressFile(tempSaveFile);
+
+	copyFile(tempSaveFile, saveFile);
+
+	remove(tempSaveFile);
 
 	updateSaveFileIndex(slot);
-
-	playTimeString = getPlayTimeAsString();
-
-	printf("Play Time : %s\n", playTimeString);
-
-	free(playTimeString);
 }
 
 static void updateSaveFileIndex(int slot)
@@ -597,12 +787,17 @@ static void updateSaveFileIndex(int slot)
 	FILE *fp;
 	int i;
 
-	time (&rawtime);
+	if (slot == -1)
+	{
+		return;
+	}
+
+	time(&rawtime);
 	timeinfo = localtime(&rawtime);
 
 	strftime(buffer, MAX_VALUE_LENGTH, "%H:%M %d %b %Y", timeinfo);
 
-	snprintf(saveName, MAX_VALUE_LENGTH, "%s - %s", getMapName(), buffer);
+	snprintf(saveName, MAX_VALUE_LENGTH, "%s - %s", _(getMapName()), buffer);
 
 	STRNCPY(data[slot], saveName, MAX_VALUE_LENGTH);
 
@@ -623,6 +818,44 @@ static void updateSaveFileIndex(int slot)
 	fclose(fp);
 }
 
+int getMostRecentSave()
+{
+	char saveFile[MAX_PATH_LENGTH];
+	struct stat fileInfo;
+	int i, modTime, slot;
+	FILE *fp;
+
+	modTime = 0;
+
+	slot = -1;
+
+	for (i=0;i<MAX_SAVE_SLOTS;i++)
+	{
+		snprintf(saveFile, sizeof(saveFile), "%ssave%d", gameSavePath, i);
+
+		fp = fopen(saveFile, "rb");
+
+		if (fp == NULL)
+		{
+			continue;
+		}
+
+		if (stat(saveFile, &fileInfo) != -1)
+		{
+			if (fileInfo.st_mtime > modTime)
+			{
+				modTime = fileInfo.st_mtime;
+
+				slot = i;
+			}
+		}
+
+		fclose(fp);
+	}
+
+	return slot;
+}
+
 void saveTemporaryData()
 {
 	char *line, itemName[MAX_MESSAGE_LENGTH], *savePtr;
@@ -633,18 +866,33 @@ void saveTemporaryData()
 	FILE *read;
 	FILE *write;
 
+	savePtr = NULL;
+
 	snprintf(swapFile, sizeof(swapFile), "%sswap", gameSavePath);
 
 	read = fopen(tempFile, "rb");
 
 	write = fopen(swapFile, "wb");
 
-	if (read != NULL)
+	if (read == NULL)
+	{
+		if (temporaryDataExists == TRUE)
+		{
+			showErrorAndExit("SAVE TEMP DATA: Could not find persistance file: %s", strerror(errno));
+		}
+	}
+
+	else
 	{
 		fclose(read);
 
 		buffer = decompressFile(tempFile);
 
+		if (strlen((char *)buffer) == 0)
+		{
+			showErrorAndExit("Something went wrong when decompressing the persistance file");
+		}
+
 		line = strtok_r((char *)buffer, "\n", &savePtr);
 
 		while (line != NULL)
@@ -665,7 +913,7 @@ void saveTemporaryData()
 
 				if (strcmpignorecase("PLAYER_DATA", line) == 0 || strcmpignorecase("PLAYER_INVENTORY", line) == 0 ||
 					strcmpignorecase("GLOBAL_TRIGGER", line) == 0 || strcmpignorecase("OBJECTIVE", line) == 0 ||
-					strcmpignorecase("PLAYER_LOCATION", line) == 0)
+					strcmpignorecase("PLAYER_LOCATION", line) == 0 || strcmpignorecase("MAP_TRIGGER", line) == 0)
 				{
 					skipping = TRUE;
 				}
@@ -715,9 +963,7 @@ void saveTemporaryData()
 
 		if (remove(tempFile) != 0)
 		{
-			perror("Could not remove temporary file");
-
-			exit(1);
+			showErrorAndExit("Could not remove temporary file");
 		}
 	}
 
@@ -741,9 +987,7 @@ void saveTemporaryData()
 
 	if (rename(swapFile, tempFile) != 0)
 	{
-		perror("Could not rename temporary file");
-
-		exit(1);
+		showErrorAndExit("Could not rename temporary file");
 	}
 
 	#if DEV == 1
@@ -751,6 +995,321 @@ void saveTemporaryData()
 	#endif
 
 	compressFile(tempFile);
+
+	temporaryDataExists = TRUE;
+}
+
+void saveContinueData()
+{
+	char itemName[MAX_MESSAGE_LENGTH], *line, *savePtr;
+	char saveFile[MAX_PATH_LENGTH];
+	char *mapName = getMapFilename();
+	unsigned char *buffer;
+	int skipping = FALSE;
+	FILE *read;
+	FILE *write;
+
+	savePtr = NULL;
+
+	snprintf(saveFile, sizeof(saveFile), "%scontinuesave", gameSavePath);
+
+	write = fopen(saveFile, "wb");
+
+	fprintf(write, "VERSION %0.2f\n", VERSION);
+
+	fprintf(write, "PLAY_TIME %ld\n", game.playTime);
+
+	fprintf(write, "PLAYER_KILLS %d\n", game.kills);
+
+	fprintf(write, "BATS_DROWNED %d\n", game.batsDrowned);
+
+	fprintf(write, "TIMES_EATEN %d\n", game.timesEaten);
+
+	fprintf(write, "DISTANCE_TRAVELLED %u\n", game.distanceTravelled);
+
+	fprintf(write, "ATTACKS_BLOCKED %d\n", game.attacksBlocked);
+
+	fprintf(write, "SLIME_TIME %d\n", game.timeSpentAsSlime);
+
+	fprintf(write, "ARROWS_FIRED %d\n", game.arrowsFired);
+
+	fprintf(write, "SECRETS_FOUND %d\n", game.secretsFound);
+
+	fprintf(write, "CONTINUES %d\n", game.continues);
+
+	fprintf(write, "CHEATING %d\n", game.cheating);
+
+	fprintf(write, "PLAYER_LOCATION %s\n", mapName);
+
+	read = fopen(tempFile, "rb");
+
+	if (read == NULL)
+	{
+		if (temporaryDataExists == TRUE)
+		{
+			showErrorAndExit("SAVE CONTINUE DATA: Could not find persistance file: %s", strerror(errno));
+		}
+	}
+
+	else
+	{
+		fclose(read);
+
+		buffer = decompressFile(tempFile);
+
+		line = strtok_r((char *)buffer, "\n", &savePtr);
+
+		while (line != NULL)
+		{
+			if (line[strlen(line) - 1] == '\n')
+			{
+				line[strlen(line) - 1] = '\0';
+			}
+
+			if (line[strlen(line) - 1] == '\r')
+			{
+				line[strlen(line) - 1] = '\0';
+			}
+
+			if (skipping == FALSE)
+			{
+				sscanf(line, "%s", itemName);
+
+				if (strcmpignorecase("PLAYER_DATA", itemName) == 0 || strcmpignorecase("PLAYER_INVENTORY", itemName) == 0 ||
+					strcmpignorecase("PLAYER_LOCATION", itemName) == 0 || strcmpignorecase("VERSION", itemName) == 0)
+				{
+					skipping = TRUE;
+				}
+
+				else if (strcmpignorecase("MAP_NAME", itemName) == 0)
+				{
+					sscanf(line, "%*s %s\n", itemName);
+
+					if (strcmpignorecase(itemName, mapName) == 0)
+					{
+						skipping = TRUE;
+					}
+
+					else
+					{
+						fprintf(write, "%s\n", line);
+					}
+				}
+
+				else
+				{
+					fprintf(write, "%s\n", line);
+				}
+			}
+
+			else
+			{
+				sscanf(line, "%s", itemName);
+
+				if (strcmpignorecase("MAP_NAME", itemName) == 0)
+				{
+					sscanf(line, "%*s %s\n", itemName);
+
+					if (strcmpignorecase(itemName, mapName) != 0)
+					{
+						skipping = FALSE;
+
+						fprintf(write, "%s\n", line);
+					}
+				}
+			}
+
+			line = strtok_r(NULL, "\n", &savePtr);
+		}
+
+		free(buffer);
+	}
+
+	/* Save the player's position */
+
+	fprintf(write, "MAP_NAME %s\n", mapName);
+
+	fprintf(write, "PLAYER_DATA\n");
+
+	writePlayerToFile(write);
+
+	fprintf(write, "INVENTORY_INDEX %d\n", getInventoryIndex());
+
+	fprintf(write, "PLAYER_INVENTORY\n");
+
+	writeInventoryToFile(write);
+
+	fprintf(write, "ENTITY_DATA\n");
+
+	/* Now write out all of the Entities */
+
+	writeEntitiesToFile(write);
+
+	/* Now the targets */
+
+	writeTargetsToFile(write);
+
+	/* And the triggers */
+
+	writeTriggersToFile(write);
+
+	/* Add the global triggers */
+
+	writeGlobalTriggersToFile(write);
+
+	/* Add the map triggers */
+
+	writeMapTriggersToFile(write);
+
+	/* Add the objectives */
+
+	writeObjectivesToFile(write);
+
+	/* Save the player data */
+
+	fclose(write);
+
+	#if DEV == 1
+		copyFile(saveFile, "continuedata");
+	#endif
+
+	compressFile(saveFile);
+}
+
+int loadContinueData()
+{
+	char itemName[MAX_MESSAGE_LENGTH], mapName[MAX_MESSAGE_LENGTH];
+	char saveFile[MAX_PATH_LENGTH], *line, *savePtr;
+	unsigned char *buffer;
+	FILE *fp;
+
+	savePtr = NULL;
+
+	snprintf(saveFile, sizeof(saveFile), "%scontinuesave", gameSavePath);
+
+	fp = fopen(saveFile, "rb");
+
+	if (fp == NULL)
+	{
+		return FALSE;
+	}
+
+	fclose(fp);
+
+	freeGameResources();
+
+	initGame();
+
+	game.canContinue = TRUE;
+
+	buffer = decompressFile(saveFile);
+
+	if (strlen((char *)buffer) == 0)
+	{
+		showErrorAndExit("Something went wrong when decompressing the continue file");
+	}
+
+	line = strtok_r((char *)buffer, "\n", &savePtr);
+
+	while (line != NULL)
+	{
+		if (line[strlen(line) - 1] == '\n')
+		{
+			line[strlen(line) - 1] = '\0';
+		}
+
+		if (line[strlen(line) - 1] == '\r')
+		{
+			line[strlen(line) - 1] = '\0';
+		}
+
+		sscanf(line, "%s", itemName);
+
+		if (strcmpignorecase("PLAY_TIME", itemName) == 0)
+		{
+			sscanf(line, "%*s %ld\n", &game.playTime);
+		}
+
+		else if (strcmpignorecase("PLAYER_KILLS", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.kills);
+		}
+
+		else if (strcmpignorecase("BATS_DROWNED", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.batsDrowned);
+		}
+
+		else if (strcmpignorecase("TIMES_EATEN", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.timesEaten);
+		}
+
+		else if (strcmpignorecase("DISTANCE_TRAVELLED", itemName) == 0)
+		{
+			sscanf(line, "%*s %u\n", &game.distanceTravelled);
+		}
+
+		else if (strcmpignorecase("ATTACKS_BLOCKED", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.attacksBlocked);
+		}
+
+		else if (strcmpignorecase("SLIME_TIME", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.timeSpentAsSlime);
+		}
+
+		else if (strcmpignorecase("ARROWS_FIRED", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.arrowsFired);
+		}
+
+		else if (strcmpignorecase("SECRETS_FOUND", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.secretsFound);
+		}
+
+		else if (strcmpignorecase("CONTINUES", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.continues);
+		}
+
+		else if (strcmpignorecase("CHEATING", itemName) == 0)
+		{
+			sscanf(line, "%*s %d\n", &game.cheating);
+		}
+
+		else if (strcmpignorecase("PLAYER_LOCATION", itemName) == 0)
+		{
+			sscanf(line, "%*s %s\n", itemName);
+
+			loadMap(itemName, FALSE);
+
+			snprintf(mapName, sizeof(mapName), "MAP_NAME %s", itemName);
+		}
+
+		else if (strcmpignorecase(line, mapName) == 0)
+		{
+			loadResources(savePtr);
+		}
+
+		line = strtok_r(NULL, "\n", &savePtr);
+	}
+
+	free(buffer);
+
+	copyFile(saveFile, tempFile);
+
+	buffer = decompressFile(tempFile);
+
+	free(buffer);
+
+	cameraSnapToTargetEntity();
+
+	freeMessageQueue();
+
+	return TRUE;
 }
 
 int hasPersistance(char *mapName)
@@ -760,13 +1319,18 @@ int hasPersistance(char *mapName)
 	unsigned char *buffer;
 	FILE *read;
 
+	savePtr = NULL;
+
 	snprintf(itemName, sizeof(itemName), "MAP_NAME %s", mapName);
 
 	read = fopen(tempFile, "rb");
 
 	if (read == NULL)
 	{
-		printf("No persistance data found\n");
+		if (temporaryDataExists == TRUE)
+		{
+			showErrorAndExit("HAS PERSISTANCE: Could not find persistance file: %s", strerror(errno));
+		}
 
 		return val;
 	}
@@ -792,6 +1356,62 @@ int hasPersistance(char *mapName)
 		if (strcmpignorecase(line, itemName) == 0)
 		{
 			val = TRUE;
+
+			break;
+		}
+
+		line = strtok_r(NULL, "\n", &savePtr);
+	}
+
+	free(buffer);
+
+	return val;
+}
+
+int bossExists(char *bossName)
+{
+	int val = FALSE;
+	char *line, *savePtr;
+	unsigned char *buffer;
+	FILE *read;
+
+	savePtr = NULL;
+
+	read = fopen(tempFile, "rb");
+
+	if (read == NULL)
+	{
+		if (temporaryDataExists == TRUE)
+		{
+			showErrorAndExit("BOSS EXISTS: Could not find persistance file: %s", strerror(errno));
+		}
+
+		return val;
+	}
+
+	fclose(read);
+
+	buffer = decompressFile(tempFile);
+
+	line = strtok_r((char *)buffer, "\n", &savePtr);
+
+	while (line != NULL)
+	{
+		if (line[strlen(line) - 1] == '\n')
+		{
+			line[strlen(line) - 1] = '\0';
+		}
+
+		if (line[strlen(line) - 1] == '\r')
+		{
+			line[strlen(line) - 1] = '\0';
+		}
+
+		if (strstr(line, bossName) != NULL)
+		{
+			val = TRUE;
+
+			break;
 		}
 
 		line = strtok_r(NULL, "\n", &savePtr);
@@ -808,6 +1428,8 @@ void loadPersitanceData(char *mapName)
 	int found = FALSE;
 	unsigned char *buffer;
 
+	savePtr = NULL;
+
 	snprintf(itemName, sizeof(itemName), "MAP_NAME %s", mapName);
 
 	buffer = decompressFile(tempFile);
@@ -830,8 +1452,6 @@ void loadPersitanceData(char *mapName)
 		{
 			found = TRUE;
 
-			printf("Loading peristance data for: %s\n", itemName);
-
 			loadResources(savePtr);
 		}
 
@@ -842,9 +1462,7 @@ void loadPersitanceData(char *mapName)
 
 	if (found == FALSE)
 	{
-		printf("Failed to find persistance data!\n");
-
-		exit(1);
+		showErrorAndExit("Failed to find persistance data!");
 	}
 }
 
@@ -860,11 +1478,23 @@ static void removeTemporaryData()
 
 		if (remove(tempFile) != 0)
 		{
-			perror("Could not remove temporary file");
-
-			exit(1);
+			showErrorAndExit("Could not remove temporary file: %s", strerror(errno));
 		}
 	}
+
+	fp = fopen(continueFile, "rb");
+
+	if (fp != NULL)
+	{
+		fclose(fp);
+
+		if (remove(continueFile) != 0)
+		{
+			showErrorAndExit("Could not remove continue file: %s", strerror(errno));
+		}
+	}
+
+	temporaryDataExists = FALSE;
 }
 
 void loadConfig()
@@ -874,6 +1504,8 @@ void loadConfig()
 	unsigned char *buffer;
 	long length;
 
+	resetGameSettings();
+
 	game.hasConfig = FALSE;
 
 	snprintf(settingsFile, sizeof(settingsFile), "%sconfig", gameSavePath);
@@ -884,20 +1516,22 @@ void loadConfig()
 
 	if (fp == NULL)
 	{
-		resetGameSettings();
+		game.firstRun = TRUE;
 
 		return;
 	}
 
+	game.firstRun = FALSE;
+
 	fseek(fp, 0L, SEEK_END);
 
 	length = ftell(fp);
 
-	buffer = (unsigned char *)malloc((length + 1) * sizeof(unsigned char));
+	buffer = malloc((length + 1) * sizeof(unsigned char));
 
 	if (buffer == NULL)
 	{
-		printf("Failed to allocate a whole %ld bytes for config file...\n", length);
+		showErrorAndExit("Failed to allocate a whole %ld bytes for config file...", length);
 
 		exit(1);
 	}
@@ -910,6 +1544,8 @@ void loadConfig()
 
 	fclose(fp);
 
+	savePtr = NULL;
+
 	line = strtok_r((char *)buffer, "\n", &savePtr);
 
 	while (line != NULL)
@@ -943,7 +1579,7 @@ void saveConfig()
 
 	if (fp == NULL)
 	{
-		perror("Could not save settings");
+		showErrorAndExit("Could not save settings: %s", strerror(errno));
 
 		exit(1);
 	}
@@ -957,17 +1593,25 @@ void saveConfig()
 
 static void copyFile(char *src, char *dest)
 {
-	char c;
+	int c;
 	FILE *sourceFile, *destFile;
 
 	sourceFile = fopen(src, "rb");
 
+	if (sourceFile == NULL)
+	{
+		showErrorAndExit("Could not open %s for reading: %s", src, strerror(errno));
+	}
+
 	destFile = fopen(dest, "wb");
 
-	while (!feof(sourceFile))
+	if (destFile == NULL)
 	{
-		c = fgetc(sourceFile);
+		showErrorAndExit("Could not open %s for writing: %s", dest, strerror(errno));
+	}
 
+	while ((c = fgetc(sourceFile)) != EOF)
+	{
 		fputc(c, destFile);
 	}
 
@@ -987,22 +1631,22 @@ char **getSaveFileIndex()
 	FILE *fp;
 	char **entries, save0[MAX_PATH_LENGTH];
 
-	entries = (char **)malloc(sizeof(char *) * MAX_SAVE_SLOTS);
+	entries = malloc(sizeof(char *) * MAX_SAVE_SLOTS);
 
 	if (entries == NULL)
 	{
-		printf("Failed to allocate a whole %d bytes for save data\n", (int)sizeof(char *) * MAX_SAVE_SLOTS);
+		showErrorAndExit("Failed to allocate a whole %d bytes for save data", (int)sizeof(char *) * MAX_SAVE_SLOTS);
 
 		exit(1);
 	}
 
 	for (i=0;i<MAX_SAVE_SLOTS;i++)
 	{
-		entries[i] = (char *)malloc(sizeof(char *) * MAX_PATH_LENGTH);
+		entries[i] = malloc(sizeof(char) * MAX_PATH_LENGTH);
 
 		if (entries[i] == NULL)
 		{
-			printf("Failed to allocate a whole %d bytes for save data\n", (int)sizeof(char) * MAX_PATH_LENGTH);
+			showErrorAndExit("Failed to allocate a whole %d bytes for save data", (int)sizeof(char) * MAX_PATH_LENGTH);
 
 			exit(1);
 		}
@@ -1047,3 +1691,117 @@ char **getSaveFileIndex()
 
 	return entries;
 }
+
+static void showPatchMessage(char *message)
+{
+	int y;
+	SDL_Rect dest;
+	SDL_Surface *title, *completion;
+
+	title = generateTextSurface(_("Patching your save game. Please wait..."), game.font, 220, 220, 220, 0, 0, 0);
+
+	completion = generateTextSurface(message, game.font, 220, 220, 220, 0, 0, 0);
+
+	if (title == NULL || completion == NULL)
+	{
+		return;
+	}
+
+	clearScreen(0, 0, 0);
+
+	y = (SCREEN_HEIGHT - title->h) / 2;
+
+	dest.x = (SCREEN_WIDTH - title->w) / 2;
+	dest.y = y;
+	dest.w = title->w;
+	dest.h = title->h;
+
+	SDL_BlitSurface(title, NULL, game.screen, &dest);
+
+	y += title->h + 15;
+
+	dest.x = (SCREEN_WIDTH - completion->w) / 2;
+	dest.y = y;
+	dest.w = completion->w;
+	dest.h = completion->h;
+
+	SDL_BlitSurface(completion, NULL, game.screen, &dest);
+
+	SDL_FreeSurface(title);
+
+	SDL_FreeSurface(completion);
+
+	/* Swap the buffers */
+
+	SDL_Flip(game.screen);
+}
+
+void loadObtainedMedals()
+{
+	char medalFile[MAX_PATH_LENGTH], *line, *savePtr;
+	unsigned char *buffer;
+	FILE *fp;
+
+	savePtr = NULL;
+
+	snprintf(medalFile, MAX_PATH_LENGTH, "%smedals", gameSavePath);
+
+	fp = fopen(medalFile, "rb");
+
+	if (fp == NULL)
+	{
+		return;
+	}
+
+	fclose(fp);
+
+	buffer = decompressFile(medalFile);
+
+	line = strtok_r((char *)buffer, "\n", &savePtr);
+
+	while (line != NULL)
+	{
+		setObtainedMedal(line);
+
+		line = strtok_r(NULL, "\n", &savePtr);
+	}
+
+	free(buffer);
+}
+
+void saveObtainedMedals()
+{
+	int i, medalCount;
+	char medalFile[MAX_PATH_LENGTH];
+	Medal *medal;
+	FILE *fp;
+
+	snprintf(medalFile, MAX_PATH_LENGTH, "%smedals", gameSavePath);
+
+	fp = fopen(medalFile, "wb");
+
+	if (fp == NULL)
+	{
+		showErrorAndExit("Could not save Medal data: %s", strerror(errno));
+	}
+
+	medalCount = getMedalCount();
+
+	medal = getMedals();
+
+	for (i=0;i<medalCount;i++)
+	{
+		if (medal[i].obtained == TRUE)
+		{
+			fprintf(fp, "%s\n", medal[i].code);
+		}
+	}
+
+	fclose(fp);
+
+	#if DEV == 1
+		copyFile(medalFile, "medaldata");
+	#endif
+
+	compressFile(medalFile);
+}
