@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2009 Parallel Realities
+Copyright (C) 2009-2016 Parallel Realities
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
 
 void saveGame(int);
@@ -28,3 +28,12 @@ void loadConfig(void);
 void newGame(void);
 char *getGameSavePath(void);
 char **getSaveFileIndex(void);
+void tutorial(void);
+int loadContinueData(void);
+void saveContinueData(void);
+int bossExists(char *);
+void startOnMap(char *);
+void loadObtainedMedals(void);
+void saveObtainedMedals(void);
+void titleScreen(void);
+int getMostRecentSave(void);
